<?php

namespace hji\membership\advancedOptions;

use hji\membership\Membership;
use hji\ResponsiveIDX\models\MarketMetadata;

class ListingStatusesOptionsSubSection implements AdvancedOptionsInterface
{
    private $market;

    private $descriptions;

    private $icon;

    private $params;

    public function __construct(string $market, string $descriptions, string $icon, array $params = [])
    {
        $this->market = $market;
        $this->descriptions = $descriptions;
        $this->icon = $icon;
        $this->params = $params;
        if (empty($this->params['heading'])) {
            $this->params['heading'] = strtoupper($this->market) . ' Options';
        }
    }

    /**
     * @inheritDoc
     */
    public function getTitle()
    {
        return strtoupper($this->market);
    }

    /**
     * @inheritDoc
     */
    public function getDescription()
    {
        return $this->descriptions;
    }

    /**
     * @inheritDoc
     */
    public function getIcon()
    {
        return $this->icon;
    }

    /**
     * @inheritDoc
     */
    public function getFields()
    {
        return $this->_getFields($this->market);
    }

    private function _getFields($market)
    {
        $globalListingOptions = Membership::getInstance()::settings()->getListingFilteringOptions();
        $listingTypes = MarketMetadata::getInstance($market)->getListingTypes();
        $statusesByTypes = MarketMetadata::getInstance($market)->getStatuses();
        $resultStatuses = [];
        if (is_array($statusesByTypes) && count($statusesByTypes) > 0) {
            foreach ($statusesByTypes as $statuses) {
                if (!is_array($statuses)) {
                    continue;
                }

                $resultStatuses = array_merge($resultStatuses, $statuses);
            }
        }

        $defaultValues = [];
        foreach ($resultStatuses as $key => $status) {
            if (strpos(strtolower($key), 'pending') === false) {
                $defaultValues[$key] = 1;
            } else {
                $defaultValues[$key] = 0;
            }
        }

        $fields = [
            [
                'id' => $market . '-broker-of-record',
                'type' => 'section',
                'title' => 'MLS Compliance',
                'subtitle' => '',
                'indent'  => true,
            ],
            [
                'id' => $market . '-broker-of-record-item',
                'type' => 'textarea',
                'title' => 'Broker of Record',
                'validate' => ['no_html'],
            ],
            [
                'id' => $market . '_listing-statuses-section',
                'type' => 'section',
                'title' => 'All Listing Types',
                'subtitle' => '',
                'indent'  => true,
            ],
            [
                'id' => $market . '_all_listing-statuses',
                'type' => 'checkbox',
                'title' => 'Status',
                'subtitle' => 'Select status filters you would like to support.<br>If nothing selected, all status filters will be supported by default.',
                'options' => $resultStatuses,
                'default' => $defaultValues
            ],
        ];

        $fields[] = [
            'id' => $market . '_all_listing-type-section_end',
            'type' => 'section',
            'title' => '',
            'subtitle' => '',
            'indent'  => false,
        ];
        foreach ($listingTypes as $key => $listingType) {
            $listingTypeProperties = MarketMetadata::getInstance($market)->_getPropertyTypesByListingType($key);

            if (isset($globalListingOptions['global-listing-types'][$key]) && $globalListingOptions['global-listing-types'][$key] === '0') {
                continue;
            }

            $fields[] = [
                'id' => $market . '_' . $key .'_listing-type-switch',
                'type' => 'switch',
                'title' => $listingType,
                'subtitle' => '',
                'default'  => '1',
            ];

            $fields[] = [
                'id' => $market . '_' . $key . '_listing-type-property',
                'type' => 'checkbox',
                'title' => 'Property Type',
                'subtitle' => 'Select property type filters you would like to support.<br>If nothing selected, all property type filters will be supported by default.',
                'options' => $listingTypeProperties,
                'required'    => [
                    $market . '_' . $key .'_listing-type-switch',
                    '=',
                    ['1']
                ],
            ];
        }

        return $fields;
    }

    public function getParams(): array
    {
        return $this->params;
    }

    /**
     * @inheritDoc
     */
    public function addActions()
    {
        add_action('redux/options/advanced_options/saved', [$this, 'saveOptions'], 10, 3);
        add_action('redux/options/advanced_options/reset', [$this, 'resetOptions'], 10, 1);
        add_action('redux/options/advanced_options/section/reset', [$this, 'resetOptions'], 10, 1);
    }

    public function resetOptions(\Redux_Panel $options): void
    {
        $markets = Membership::getInstance()->getMarkets();
        sort($markets);

        $allFields = [];
        $fields = [];
        foreach ($markets as $market) {
            $allFields = array_merge($allFields, $this->_getFields($market));
        }

        foreach ($allFields as $value) {
            if ($value['type'] === 'section') {
                continue;
            }

            if (!empty($value['options']) && is_array($value['options'])) {
                $fields[$value['id']] = array_fill_keys(array_keys($value['options']), '');
            } else {
                $fields[$value['id']] = $value['default'] ?? '';
            }
        }

        foreach ($fields as $key => $value) {
            if (
                isset($options->parent->options[$key])
                && gettype($value) === gettype($options->parent->options[$key])
            ) {
                $fields[$key] = $options->parent->options[$key];
            }
        }

        $this->saveOptions($fields, true);
    }

    /**
     * @param array $values
     */
    public function saveOptions($values, $reset = false): void
    {
        $valueTemp = $values;
        unset($valueTemp['last_tab']);
        unset($valueTemp['lookupAreaList']);
        unset($valueTemp['areaBias']);
        unset($valueTemp['lookupAreaDefaultPlaceholder']);
        unset($valueTemp['licensedInStates']);

        $options = Membership::getInstance()::settings()->getListingFilteringOptions();
        foreach ($valueTemp as $key => $value) {
            $keyParts = explode('_', $key);
            if ($keyParts && count($keyParts) === 3) {
                switch ($keyParts[2]) {
                    case 'listing-type-switch': if ($keyParts[0] !== 'global') {
                        $options['markets'][$keyParts[0]]['listing-types'][$keyParts[1]] = $value;
                    }
                        break;
                    case 'listing-type-property':
                        if (is_array($value)) {
                            foreach ($value as $listingProperty => $listingProperyValue) {
                                $options['markets'][$keyParts[0]]['property-types'][$keyParts[1]][$listingProperty] = $listingProperyValue;
                            }
                        }
                        break;
                    case 'listing-statuses':
                        if (is_array($value)) {
                            foreach ($value as $listingStatusKey => $listingStatusValue) {
                                $options['markets'][$keyParts[0]]['listing-statuses'][$listingStatusKey] = $listingStatusValue;
                            }
                        }
                        break;
                }
            }
        }

        Membership::getInstance()::settings()->setGlobalListingOptions($options);
    }
}
