<?php

namespace hji\membership\controllers;

use hji\membership\includes\CustomerInfoSynchronizer;
use hji\membership\Membership;

use hji\membership\models\APIClientModel;
use hji\membership\models\Customer;
use hji\membership\models\UserSession;
use hji\membership\utils\Environment;


class APIClient
{
    private $customerModel;
    private $settingsModel;
    private $licenseKey;
    private $mustUpdateCustomerInfo = true;

    /**
     * @var APIClientModel
     *
     */
    public $api;


    function __construct(Customer $customer)
    {
        $this->customerModel = Customer::getInstance();
        $this->settingsModel = Membership::settings();
        $this->licenseKey    = $this->settingsModel->licenseKey;

        if (!is_object($this->api))
        {
            $this->createAPIinstance();
        }

        // Check if authentication is forced via $_GET

        if (isset($_GET['hjauth']) && $_GET['hjauth'] == 'force')
        {
            // Since authentication method can be called numerous times,
            // it has to be forced only ones by the GET method.
            // Instead of unsetting it, I'm changing value for debugging purposes.

            $_GET['hjauth'] = 'forced once';
            $forceAuthentication = true;
        }
        else
        {
            $forceAuthentication = false;
        }

        if ($forceAuthentication)
        {
            $this->authenticate();
        }

        add_action('wp_ajax_hjiAjaxApiRequest', array($this, 'ajaxApiRequest'));
        add_action('wp_ajax_nopriv_hjiAjaxApiRequest', array($this, 'ajaxApiRequest'));

    }

    /**
     * @return bool
     */
    public function getMustUpdateCustomerInfo()
    {
        return $this->mustUpdateCustomerInfo;
    }

    /**
     * @param bool $value
     * @return $this
     */
    public function setMustUpdateCustomerInfo($value)
    {
        $this->mustUpdateCustomerInfo = $value;

        return $this;
    }


    function createAPIinstance($mustUpdateCustomerInfo = true)
    {
        // Instance of the api client model

        $api = new APIClientModel(Membership::getInstance(), $this);
        $api::$areaBias = $this->settingsModel->areaBias;
        $this->api = $api;
        $this->setMustUpdateCustomerInfo($mustUpdateCustomerInfo);

        // Set cache transport

        $this->api->setCacheTransport(new \hji\common\utils\WPCache());

        // Set dev/debug modes

        if (Environment::is(Environment::DEV))
        {
            $this->api->setDeveloperMode(true);
            $this->api->setDebugMode(true);
        }

        // Update API object with existing token information

        $this->api->last_token          = UserSession::getToken();
        $this->api->last_token_expires  = UserSession::getTokenExpires();
    }


    function authenticate($force = false, $params = array())
    {
        // Since we're re-authentication, current user session needs to be destroyed
        UserSession::destroy();

        $result = ($this->licenseKey) ? $this->api->authenticate($force, $params) : false;
        $success = false;

        if ($result && $result->success && $result->result)
        {
            UserSession::start($result->result->token, $result->result->expires);
            $success = true;
        }

        if (isset($result->result)) {
            $this->customerModel->update(@$result->result);
        } else {
            $this->customerModel->update(false);
        }

        if ($this->mustUpdateCustomerInfo) {
            (new CustomerInfoSynchronizer())
                ->synchronizeSlipstreamToSite($this->customerModel->customer, $this->settingsModel->customer)
            ;
        }

        // Update API object with new token information

        $this->api->last_token          = UserSession::getToken();
        $this->api->last_token_expires  = UserSession::getTokenExpires();

        if ($this->api->getErrors())
        {
            add_action('admin_notices', array($this, 'authFailureNotice'));
        }

        return $success;
    }


    function authFailureNotice()
    {
        echo '<div id="message" class="error"><p><strong>' . $this->api->getErrors() . '</strong> Check your <a href="' . admin_url('admin.php?page=' . Membership::$slug) . '">' . Membership::NAME . ' Settings</a> or contact customer support.</p></div>';
    }


    /** AJAX API Methods */

    /**
     * Bridge between JS and PHP API methods
     * Pass method and its parameters
     *
     * @param string method
     * @param array params
     *
     * @return json encoded array
     */
    function ajaxApiRequest()
    {
        if ((!isset($_POST['method']) || empty($_POST['params'])))
        {
            die(-1);
        }

        $method = trim($_POST['method']);

        if (!method_exists($this->api, $method))
        {
            $response = array(
                'success'   => false,
                'error'     => array(
                    'message' => 'API Method ' . $method . ' doesn\'t exist'
                )
            );

            die(json_encode($response));
        }

        // If any of the params are stringified json, it needs to be re-encoded

        $params = (array)$_POST['params'];

        foreach ((array)$_POST['params'] as $k => $v)
        {
            if (empty($v))
            {
                unset($params[$k]);
                continue;
            }

            if (is_string($v) && !ctype_digit($v))
            {
                $tmp = $v;
                $isJSON = stripslashes($tmp);
                $json = json_decode($isJSON);

                if (json_last_error() == JSON_ERROR_NONE)
                {
                    $params[$k] = json_encode($json);
                }
                else
                {
                    $params[$k] = $v;
                }
            }
            else
            {
                $params[$k] = $v;
            }
        }

        $results = $this->api->$method($params);

        if ($results && !$this->api->getErrors())
        {
            $response = array(
                'success'   => true,
                'data'     => array(
                    $results
                )
            );
            die(json_encode($response));
        }
        else if ($error = $this->api->getErrors())
        {
            $response = array(
                'success'   => false,
                'error'     => array(
                    'message' => $error
                )
            );

            die(json_encode($response));
        }

        $response = array(
            'success'   => false,
            'error'     => array(
                'message' => 'Request failed.'
            )
        );

        die(json_encode($response));
    }


    /**
     * Deprecated since 1.0.4
     *
     * returning instance via Membership
     */
    static function getInstance()
    {
        _deprecated_function('getInstance', '1.0.4', 'Membership::$apiController');

        $membership = Membership::getInstance();

        return $membership->apiController;
    }
}
