<?php

namespace hji\membership\extension\postTypes;


use hji\common\utils\MetaBoxes;
use hji\membership\Membership;


/**
 * Class PostTypeManager
 *
 * @package hji\membership\extension\postTypes
 *
 *  What does it do?
 *      1. Registers Post Type with WP
 *      2. Registers Post Type Metaboxes with WP
 *      3. Registers Post Type Metafields with WP Rest API
 *      4. Registers Post Type Taxonomies with WP
 */
class PostTypeManager
{
    protected $factory;

    protected $postTypeClasses = [];

    protected static $postTypes = [];

    /**
     * PostTypeManager constructor.
     *
     * @param Factory $factory
     */
    public function __construct(Factory $factory)
    {
        $this->factory = $factory;

        if (!has_action('init', [$this, 'registerAll']))
            add_action('init', [$this, 'registerAll']);
    }


    public function registerAll()
    {
        foreach ($this->postTypeClasses as $classReference)
        {
            $this->register($this->factory->resolvePostType($classReference));
        }
    }


    public function getPostType($name)
    {
        return self::$postTypes[$name] ?? null;
    }


    /**
     * Adds multiple objects to the factory all at once
     *
     * @param array $postTypeClasses [CustomPostType::class]
     *
     * @throws \Exception
     */
    public function addPostTypes(array $postTypeClasses)
    {
        array_map([$this, 'addPostType'], $postTypeClasses);
    }


    /**
     * Adds a post object array to the factory.
     *
     * @param string $postTypeClass::class reference
     *
     * @throws \Exception
     */
    public function addPostType(string $postTypeClass)
    {
        if (array_search($postTypeClass, (array)$this->postTypeClasses) === false)
        {
            $this->postTypeClasses[] = $postTypeClass;
        }
        else
        {
            $error = "Duplicate post type. "
                ."Post type [{$postTypeClass}] is already registered.";
            throw new \Exception($error);
        }
    }


    protected function register(PostTypeModel $postType)
    {
        self::$postTypes[$postType->name] = $postType;

        $this->registerPostType($postType);
        $this->registerPostTypeMeta($postType);
        $this->registerPostTypeRestAPI($postType);
        $this->registerPostTypeTaxonomies($postType);
    }


    protected function registerPostType($postType)
    {
        require_once Membership::$dir . '/third-party/extended-cpts/extended-cpts.php';

        $extendedCPTModel = register_extended_post_type(
            $postType->name,
            $this->getPostRegistrationArgs($postType),
            [
                'slug'     => $postType->slug,
                'plural'   => $postType->plural_name ?? null,
                'singular' => $postType->singular_name ?? null
            ]);


        return $extendedCPTModel;
    }


    protected function registerPostTypeMeta($postType)
    {
        MetaBoxes::add($postType->getMetaboxes());
    }


    protected function registerPostTypeRestAPI($postType)
    {
        if ($postType->show_in_rest === true)
        {
            $ptToRest = $this->factory->createPostTypeToRest();
            $ptToRest->setPostType($postType)
                ->registerFields();
        }
    }


    protected function registerPostTypeTaxonomies($postType)
    {
        // @codeCoverageIgnoreStart
        if (!$taxonomies = $postType->getTaxonomies())
            return;
        // @codeCoverageIgnoreEnd

        $taxonomyManager = $this->factory->createTaxonomyManager();

        foreach ($taxonomies as $taxonomy)
        {
            $taxonomyManager->registerTaxonomy($taxonomy, $postType->name);
        }
    }


    protected function getPostRegistrationArgs($postType)
    {
        $exclude = ['taxonomies']; // we register them separately
        $props   = $postType->getProperties();

        foreach ($exclude as $prop)
        {
            if (!in_array($prop, $exclude))
            {
                unset($props[$prop]);
            }
        }

        return $props;
    }
}