<?php
namespace hji\membership\includes;


use hji\common\models\Notices;
use hji\membership\Membership;
use \hji\membership\models\Settings as M_Settings;
use hji\membership\models\UserSession;


class CustomerInfoSynchronizer
{
    /**
     * @var
     */
    protected $notices;

    /**
     * @var null
     */
    protected $themeOptions = null;

    /**
     * @var array
     */
    protected $synchronizeFields = [
        'name' => '',
        'officeName' => '',
        'email' => '',
        'emailAddress' => '',
        'officePhone' => '',
        'mobilePhone' => '',
        'fax' => '',
        'photo' => '',
        'logo' => '',
        'address' => [
            'deliveryLine' => '',
            'city' => '',
            'state' => '',
            'street' => '',
            'zip' => '',
        ],
    ];

    /**
     * @param Notices $notices
     * @return void
     */
    function setNotices(Notices $notices)
    {
        $this->notices = $notices;
    }


    function setActionHooks()
    {
    }


    /**
     * Checks if customer info needs to be synced both ways
     *
     * Syncs customer data form API meta with Membership Settings
     * Syncs customer images form theme options to Slipstream
     */
    function autosyncCustomerInfo()
    {
        $settings = new M_Settings();

        // Sync only if name or email are not set, that's first obvious indicator

        if (empty($settings->customer['name'])|| empty($settings->customer['email']))
        {
            // Save customer data form API into settings

            $this->prePopulateCustomerInfo($settings);

            // Update customer images form theme options
            // if they are not set in Membership Settings.

            if ($updated = $this->updateCustomerImages())
            {
                // If images were added, sync them to Slipstream

                $this->syncCustomerInfoToSlipstream();
            }
        }
    }


    /**
     * Pushes customer data saved in Settings to Slipstream
     */
    public function syncCustomerInfoToSlipstream()
    {
        if (!Membership::getInstance()->isLicensed() )
            return;

        $settings = new M_Settings();
        $data     = $settings->all();

        if (isset($data['customer']) && !empty($data['customer']))
        {
            $this->synchronizeSiteToSlipstream($data['customer']);
        }
    }

    public function synchronizeSlipstreamToSite($slipstreamCustomerInfo, $dbCustomerInfo)
    {
        // Normalize customer info to array
        $slipstreamCustomerInfo = json_decode(json_encode($slipstreamCustomerInfo), true);

        if(!$this->checkEqualsCustomersData($slipstreamCustomerInfo, $dbCustomerInfo)) {
            $settings = new M_Settings();

            $newCustomerInfo = $dbCustomerInfo;

            $slipstreamCustomerInfo = $this->normalizeCustomerInfo($slipstreamCustomerInfo);

            foreach ($dbCustomerInfo as $key => $value) {
                if (!is_array($value)) {
                    if (isset($slipstreamCustomerInfo[$key])) {
                        $newCustomerInfo[$key] = $slipstreamCustomerInfo[$key];
                    }
                } else {
                    foreach ($value as $key2 => $value2) {
                        if (isset($slipstreamCustomerInfo[$key][$key2])) {
                            $newCustomerInfo[$key][$key2] = $slipstreamCustomerInfo[$key][$key2];
                        }
                    }
                }
            }

            $settings->updateCustomer($newCustomerInfo);
        }
    }


    /**
     * API call to sync customer data
     *
     * @param array $customer
     *
     * @return bool
     */
    protected function synchronizeSiteToSlipstream(array $customer)
    {
        $params = $this->prepareCustomerParams($customer);

        // Force-create API instance to re-construct with a fresh settings model,
        // which may have not the same license key
        Membership::getInstance()->apiController->createAPIinstance(false);
        $api = Membership::getInstance()->apiController->api;

        $api->makeAPIRequest('POST', "/ws/customers/synchronize", array(), http_build_query($params), false);

        UserSession::destroy();


        if ($error = $api->getErrors())
        {
            error_log('Customer Data Sync Failed: ' . $error);
            return false;
        }

        return true;
    }


    /**
     * Pre-populates settings model with customer info from API metadata
     *
     * @param $settings Membership Settings Model
     */
    protected function prePopulateCustomerInfo($settings)
    {
        if ($customer = Membership::getInstance()->customerModel->customer)
        {
            $customer = json_decode(json_encode($customer), true);
//            $options = $settings->all();
            $options = array_merge(
                (array)$settings->all(),
                ['customer' => (array)$customer]
            );
//            $options['customer'] = $customer;
            $settings->setOptions($options);
        }
    }


    /**
     * Prepares customer parameters for synchronization
     *
     * Flattens array into single level
     * and assigns false to null values to unset them in Slipstream
     *
     * @param $customer
     *
     * @return array
     */
    protected function prepareCustomerParams($customer)
    {
        // Local fields to Slipstream properties
        $supportedFieldsMap = array(
//            'name'          => 'name',
//            'email'         => 'email',
//            'officeName'    => 'officeName',
            'officePhone'   => 'officePhone',
            'mobilePhone'   => 'mobilePhone',
            'fax'           => 'fax',
            'deliveryLine'  => 'deliveryLine',
            'city'          => 'city',
            'state'         => 'state',
            'zip'           => 'zip',
            'photo'         => 'photo',
            'logo'          => 'logo',
            'logoHeight' => 'logoHeight'
        );

        $result = array();
        $params = $customer;

        // Flatten array of customer info parameters

        foreach ($customer as $k => $v)
        {
            if (is_array($v))
            {
                $params = array_merge($params, $this->prepareCustomerParams($v));
                unset($params[$k]);
            }
            else if (empty($v))
            {
                $params[$k] = false;
            }
        }

        // Map params to supported  Slipstream properties

        foreach ($params as $k => $v)
        {
            if (isset($supportedFieldsMap[$k]))
            {
                $result[$supportedFieldsMap[$k]] = $v;
            }
        }

        return $result;
    }


    /**
     * Updates customer images settings with corresponding items in Theme Options.
     *
     * @return bool true if images were set in settings, false otherwise.
     */
    public function updateCustomerImages()
    {
        $updated = false;
        $settings = new M_Settings();
        $options = $settings->all();

        if (empty($settings->customer['photo']) && $customerPhoto = $this->getThemeHeadshot())
        {
            $options['customer']['photo'] = $customerPhoto;
            $updated = true;
        }

        if (empty($settings->customer['logo']) && $customerLogo = $this->getThemeLogo())
        {
            $options['customer']['logo'] = $customerLogo;
            $updated = true;
        }

        if ($updated)
            $settings->setOptions($options);

        return $updated;
    }


    /**
     * Returns customer logo URL form Theme Options
     *
     * @return string|null
     */
    protected function getThemeLogo()
    {
        $themeOptions = $this->getThemeOptions();

        if (isset($themeOptions['site_logo']['id']) && !empty($themeOptions['site_logo']['id']))
        {
            if ($img = image_downsize($themeOptions['site_logo']['id'], 'hji/200h'))
            {
                return $img[0];
            }
        }
    }


    /**
     * Returns customer headshot URL form Theme Options
     *
     * @return string|null
     */
    protected function getThemeHeadshot()
    {
        $themeOptions = $this->getThemeOptions();

        if (isset($themeOptions['agent_image']['id']) && !empty($themeOptions['agent_image']['id']))
        {
            if ($img = image_downsize($themeOptions['agent_image']['id'], 'hji/150w150h'))
            {
                return $img[0];
            }
        }
    }


    /**
     * Returns Blvd Theme Options
     *
     * @return array|null
     */
    protected function getThemeOptions()
    {
        if ($this->themeOptions === null)
            $this->themeOptions = get_option('hji-boulevard-settings', array());

        return $this->themeOptions;
    }

    /**
     * @param array $slipstreamCustomerInfo
     * @param array $dbCustomerInfo
     * @return bool
     */
    private function checkEqualsCustomersData($slipstreamCustomerInfo, $dbCustomerInfo)
    {
        $slipstreamCustomerInfo = $this->normalizeCustomerInfo($slipstreamCustomerInfo);
        $dbCustomerInfo = $this->normalizeCustomerInfo($dbCustomerInfo, false);

        foreach ($this->synchronizeFields as $key => $value) {
            if (!is_array($value)) {
                if ($slipstreamCustomerInfo[$key] !== $dbCustomerInfo[$key]) {
                    return false;
                }
            } else {
                foreach ($value as $key2 => $value2) {
                    if ($slipstreamCustomerInfo[$key][$key2] !== $dbCustomerInfo[$key][$key2]) {
                        return false;
                    }
                }
            }
        }

        return true;
    }

    /**
     * @param array $data
     * @param bool $serverData
     * @return array
     */
    private function normalizeCustomerInfo($data, $serverData = true)
    {
        $result = [];

        foreach ($this->synchronizeFields as $key => $value) {
            if (!is_array($value)) {
                if (!empty($data[$key])) {
                    $result[$key] = $data[$key];
                } else {
                    $result[$key] = '';
                }
            } else {
                foreach ($value as $key2 => $value2) {
                    if (!empty($data[$key][$key2])) {
                        $result[$key][$key2] = $data[$key][$key2];
                    } elseif ($serverData) {
                        $result[$key][$key2] = '';
                    }
                }
            }
        }

        return $result;
    }
}
