<?php
namespace hji\membership\models;


use hji\common\models\ServerDataCache;

class Customer
{
    private static $_instance;

    // Option name where customer data is being saved,
    // Which is returned on license key authentication:
    // customer, products, markets etc.

    const AUTH_DATA = 'hji-membership-customer';

    public $license = null;
    public $customer = null;
    public $markets = null;

    /**
     * Data, which we stored in wordpress cache, but split data by first level property
     * @var array
     */
    protected static $dataToSplitCache = [];

    /**
     * Data, which we stored in wordpress cache, but compress data
     * @var array
     */
    protected static $dataToCompressedCache = ['markets'];

    /**
     * @var ServerDataCache
     */
    private static $serverDataCache;


    static function getInstance()
    {
        if (!(self::$_instance instanceof self)) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    /**
     * Customer constructor.
     */
    function __construct()
    {
        self::$serverDataCache = new ServerDataCache;
        $this->init();
    }

    /**
     * Init Customer object
     */
    function init()
    {
        $authData = self::$serverDataCache->getData(self::getCustomerDataId());
        $settingsData = get_option(Settings::KEY);

        if (isset($authData->license->licenseKey)) {
            $this->license = $authData->license;
        } else {
            $this->license = new \stdClass();
            $this->license->licenseKey = !empty($settingsData['license-key']) ? trim($settingsData['license-key']) : false;
            $this->license->token       = null;
            $this->license->expires     = null;
        }

        if (isset($authData->customer)) {
            $this->customer = $authData->customer;
        }

        foreach (self::$dataToSplitCache as $cacheItem) {
            $cacheData = self::$serverDataCache->getSplitData($cacheItem);
            $this->$cacheItem = ($cacheData === false) ? null : $cacheData;
        }
        foreach (self::$dataToCompressedCache as $cacheItem) {
            $cacheData = self::$serverDataCache->getCompressedData($cacheItem);
            $this->$cacheItem = ($cacheData === false) ? null : $cacheData;
        }
    }


    /**
     * Returns array of Markets licensed to Customer
     *  market ids to market names
     *
     * @return (string|string)[]|null
     */
    function getMarkets()
    {
        return ($this->markets) ? $this->markets : null;
    }


    function update($data = false)
    {
        $this->destroySessionAndServerCache();

        if (isset($data->licenseKey)
            && isset($data->token)
            && isset($data->expires))
        {
            $this->license->licenseKey  = $data->licenseKey;

            // TODO: token and its expiration is related to website visitor - not hji customer
            /// their instances should be removed.

            // This token can be use only for non browser requests (cron, etc)
            $this->license->token       = $data->token;
            $this->license->expires     = $data->expires;

            if (isset($data->customer))
            {
                $customer = new \stdClass();

                foreach ($data->customer as $k => $v)
                {
                    $customer->$k = $v;
                }

                $this->customer = $customer;
            }

            if (isset($data->markets) && is_array($data->markets))
            {
                // Convert array of markets into an object
                // where each market can be accessed by its ID
                // i.e.: $market->sandicor

                $obj = new \stdClass();

                foreach ($data->markets as $market)
                {
                    $obj->{$market->id} = $market;
                }

                $this->markets = $obj;
            }
        }
        else
        {
            $this->customer         = null;
            $this->markets          = null;
            $this->license->licenseKey   = null;
            $this->license->token   = null;
            $this->license->expires = null;
        }

        $this->updateSessionAndServerCache($this);
    }

    private function destroySessionAndServerCache()
    {
        self::$serverDataCache->unsetData(self::getCustomerDataId());
        self::$serverDataCache->unsetData(self::$dataToSplitCache);
        self::$serverDataCache->unsetData(self::$dataToCompressedCache);
    }


    private function updateSessionAndServerCache($authData)
    {
        // Convert object into standard since we can't load our User class definition
        // before session starts, otherwise it throws __PHP_Incomplete_Class
        $data = get_object_vars($authData);

        if ($data)
        {
            $authData = new \stdClass();

            foreach ($data as $k => $v) {
                if (in_array($k, self::$dataToSplitCache)) {
                    self::$serverDataCache->setSplitData($k, $v);
                } elseif (in_array($k, self::$dataToCompressedCache)) {
                    self::$serverDataCache->setCompressedData($k, $v);
                } else {
                    $authData->$k = $v;
                }
            }
            self::$serverDataCache->setData(self::getCustomerDataId(), $authData);
        }
        else
        {
            $this->destroySessionAndServerCache();
        }

        $this->init();
    }


    private static function getCustomerDataId()
    {
        return self::AUTH_DATA . get_current_blog_id();
    }
}