<?php

namespace hji\membership\models;

use \hji\membership\interfaces\Product;


class ProductDependencies
{
    const KEY_PACKAGE           = 'package';
    const KEY_REQUIRE_THEME     = 'require_theme';
    const KEY_REQUIRE_PLUGINS   = 'require_plugins';

    protected static $fileHeaders = array();

    protected static $jsonProperties = array();

    /**
     * Array of DependencyEntity objects
     *
     * @var array
     */
    protected $dependencies;

    /**
     * Array of DependencyEntity objects for product update
     *
     * @var array
     */
    protected $updateDependencies;

    /**
     * @var Product object
     */
    protected $product;


    function __construct(Product $product)
    {
        $this->product = $product;

        self::$fileHeaders = array(
            self::KEY_PACKAGE           => 'HJI Package',
            self::KEY_REQUIRE_THEME     => 'HJI Require Template',
            self::KEY_REQUIRE_PLUGINS   => 'HJI Require Plugins',
        );

        self::$jsonProperties = array(
            self::KEY_PACKAGE           => 'hji_package',
            self::KEY_REQUIRE_THEME     => 'hji_require_template',
            self::KEY_REQUIRE_PLUGINS   => 'hji_require_plugins',
        );
    }


    /**
     * Returns array of product dependencies - DependencyEntity objects
     *
     * @return array
     */
    public function getDependencies()
    {
        if (!$this->dependencies && $dependenciesInfo = $this->_getHeadersInfo())
        {
            $model = $this->createDependencyModel($dependenciesInfo);
            $this->dependencies = $model->require;
        }

        return $this->dependencies;
    }


    /**
     * Returns array of product update dependencies - DependencyEntity objects
     *
     * @return array
     */
    public function getUpdateDependencies()
    {
        if (!$this->updateDependencies && $dependenciesInfo = $this->_getUpdateInfo())
        {
            $model = $this->createDependencyModel($dependenciesInfo);
            $this->updateDependencies = $model->require;
        }

        return $this->updateDependencies;
    }


    /**
     * Determines which dependencies are not met for the current product
     *
     * @return array|bool
     */
    public function getMissingDependencies()
    {
        if ($dependencies = $this->getDependencies())
        {
            return $this->_getMissingDependencies($dependencies);
        }

        return false;
    }


    /**
     * Determines which dependencies are not met for the product update
     *
     * @return array|bool
     */
    public function getMissingUpdateDependencies()
    {
        if ($updateDependencies = $this->getUpdateDependencies())
        {
            return $this->_getMissingDependencies($updateDependencies);
        }

        return false;
    }


    /**
     * Returns array of missing dependencies or false if none
     *
     * @param array $dependencies array of DependencyEntities
     * @return array|bool
     */
    protected function _getMissingDependencies(array $dependencies)
    {
        $missingDependencies = [];

        if (!empty($dependencies)) {
            foreach ($dependencies as $requiredItem) {
                /** @var DependencyEntity  $requiredItem */
                if ($installedVersion = $this->_getInstalledVersion($requiredItem)) {
                    $requiredItem->installed = true;
                    $requiredItem->correctVersion = !version_compare($installedVersion, $requiredItem->version, '<');
                } else {
                    $requiredItem->installed = false;
                }

                if ($requiredItem instanceof DependencyPluginEntity && !is_network_admin()) {
                    $requiredItem->activated = $requiredItem->installed ? is_plugin_active($requiredItem->getPluginPath()) : false;
                }

                if (
                    !$requiredItem->installed
                    || !$requiredItem->correctVersion
                    || ($requiredItem instanceof DependencyPluginEntity && !is_network_admin() && !$requiredItem->activated)
                ) {
                    $missingDependencies[] = $requiredItem;
                }
            }
        }

        return $missingDependencies;
    }


    /**
     * Returns a version of currently installed item (plugin|theme)
     *
     * @param DependencyEntity $requiredItem
     * @return bool|string
     */
    protected function _getInstalledVersion(DependencyEntity $requiredItem)
    {
        if ($requiredItem instanceof DependencyPluginEntity)
        {
            return $requiredItem->getPluginVersion();
        }
        else if ($requiredItem->type == 'theme')
        {
            $theme = wp_get_theme($requiredItem->name);

            if ($theme->exists() && $version = $theme->get('Version'))
            {
                return $version;
            }
        }
    }


    /**
     * Returns array of header properties to be used in main plugins file
     * or main style.css file in themes
     *
     * @return array
     */
    public static function getFileHeaderProperties()
    {
        $headers = array();

        foreach (self::$fileHeaders as $fileHeader)
        {
            $headers[$fileHeader] = $fileHeader;
        }

        return $headers;
    }


    public static function getFileHeadersHashmap()
    {
        return self::$fileHeaders;
    }


    /**
     * Creates Dependency Model Object.
     *
     * @param $dependenciesInfo
     *      @prop package
     *      @prop require_theme
     *      @prop require_plugins
     * @return DependencyModel
     */
    protected function createDependencyModel($dependenciesInfo)
    {
        return new DependencyModel(
            $dependenciesInfo[self::KEY_PACKAGE],
            $dependenciesInfo[self::KEY_REQUIRE_THEME],
            $dependenciesInfo[self::KEY_REQUIRE_PLUGINS]);
    }


    protected function _getHeadersInfo()
    {
        $dependencyInfo[self::KEY_PACKAGE]         = $this->_getHeaderProperty(self::KEY_PACKAGE);
        $dependencyInfo[self::KEY_REQUIRE_THEME]   = $this->_parseHeader($this->_getHeaderProperty(self::KEY_REQUIRE_THEME));
        $dependencyInfo[self::KEY_REQUIRE_PLUGINS] = $this->_parseHeader($this->_getHeaderProperty(self::KEY_REQUIRE_PLUGINS));

        if ($dependencyInfo['package'] !== null)
        {
            return $dependencyInfo;
        }
    }


    protected function _getUpdateInfo()
    {
        $dependencyInfo[self::KEY_PACKAGE]         = $this->_getJSONProperty(self::KEY_PACKAGE);
        $dependencyInfo[self::KEY_REQUIRE_THEME]   = $this->_getJSONProperty(self::KEY_REQUIRE_THEME);
        $dependencyInfo[self::KEY_REQUIRE_PLUGINS] = $this->_getJSONProperty(self::KEY_REQUIRE_PLUGINS);

        if ($dependencyInfo['package'] !== null)
        {
            return $dependencyInfo;
        }
    }


    protected function _getHeaderProperty($key)
    {
        $data = $this->product->getData();

        return isset($data[self::$fileHeaders[$key]]) ? $data[self::$fileHeaders[$key]] : null;
    }


    protected function _getJSONProperty($key)
    {
        $updateInfo = $this->product->getUpdateInfo();

        return isset($updateInfo[self::$jsonProperties[$key]]) ? $updateInfo[self::$jsonProperties[$key]] : false;
    }


    /**
     * @param $header string
     * @return array|bool
     */
    protected function _parseHeader($header)
    {
        // HJI Membership:2.0-b15, HJI Responsive IDX:3.0-b18

        $dependencies = array();
        $parts = array_map('trim', explode(',', $header));

        if (empty($parts))
        {
            return false;
        }

        foreach ($parts as $part)
        {
            $dependency = array_map('trim', explode(':', $part));

            if (!isset($dependency[1]))
                continue;

            $dependencies[$dependency[0]] = $dependency[1];
        }

        return !empty($dependencies) ? $dependencies : false;
    }
}
