<?php

namespace hji\membership\models;

use hji\common\AbstractSettingsModel;
use hji\membership\Membership;
use hji\ResponsiveIDX\models\MarketMetadata;


class Settings extends \hji\common\interfaces\AbstractSettingsModel
{
    protected static $__CLASS__ = __CLASS__; // Provide this in each singleton class.

    /** @var string     Options handle key */
    const KEY = 'hji-membership-settings';
    const GOOGLE_OPTIONS_KEY = 'hji-google-api';
    const DEFAULT_LOGO_HEIGHT = 75;

    /** @var string     Slipstream API License Key */
    public $licenseKey;

    /**
     * @since 2.6.0
     * @see https://slipstream.homejunction.com/docs/ws/areas#bias
     * @var string      Area Bias: postal-city, census-pace, none.
     */
    public $areaBias;

    /** @var array      Customer Information **/
    public $customer;

    /**
     * @var null
     */
    protected $gfMappingOptions = null;


    public function __construct()
    {
        parent::__construct('membership', self::KEY);

        $this->loadOptions();

        $data = $this->settings;

        $this->licenseKey = !empty($data['license-key']) ? trim($data['license-key']) : null;
        $this->areaBias   = (isset($data['areaBias']) && !empty($data['areaBias'])) ? $data['areaBias'] : null;

        $this->customer['name']  =  !empty($data['customer']['name']) ? trim($data['customer']['name']) : null;
        $this->customer['email'] =  !empty($data['customer']['email']) ? trim($data['customer']['email']) : null;
        $this->customer['officeName']  =  !empty($data['customer']['officeName']) ? trim($data['customer']['officeName']) : null;
        $this->customer['officePhone'] =  !empty($data['customer']['officePhone']) ? trim($data['customer']['officePhone']) : null;
        $this->customer['mobilePhone'] =  !empty($data['customer']['mobilePhone']) ? trim($data['customer']['mobilePhone']) : null;
        $this->customer['fax'] =  !empty($data['customer']['fax']) ? trim($data['customer']['fax']) : null;
        $this->customer['address']['deliveryLine'] =  !empty($data['customer']['address']['deliveryLine']) ? trim($data['customer']['address']['deliveryLine']) : null;
        $this->customer['address']['city']  =  !empty($data['customer']['address']['city']) ? trim($data['customer']['address']['city']) : null;
        $this->customer['address']['state'] =  !empty($data['customer']['address']['state']) ? trim($data['customer']['address']['state']) : null;
        $this->customer['address']['zip']   =  !empty($data['customer']['address']['zip']) ? trim($data['customer']['address']['zip']) : null;
        $this->customer['photo'] = !empty($data['customer']['photo']) ? trim($data['customer']['photo']) : null;
        $this->customer['logo']  = !empty($data['customer']['logo']) ? trim($data['customer']['logo']) : null;
        $this->customer['logoHeight'] =
            !empty($data['customer']['logoHeight']) ? $data['customer']['logoHeight'] : self::DEFAULT_LOGO_HEIGHT
        ;

        $this->crm = $this->getCRMOptions();
        $this->googleApi = $this->getGoogleApiSettings();
    }


    /**
     * Sets Area Bias option to Census Places
     *
     * @since 2.6.0
     * @see docs/Area Bias.md
     *
     * @return $this
     */
    public function setCensusPlaceBias()
    {
        $censusPlacesBias = 'census-place';
        $this->overwriteAreaBias($censusPlacesBias);
        return $this;
    }

    /**
     * @param array $stateLicensedIn
     *
     * @return $this
     */
    public function setStateLicensedIn(array $stateLicensedIn)
    {
        $this->settings['licensedInStates'] = $stateLicensedIn;
        $this->setOptions($this->settings);
        $this->loadOptions();

        return $this;
    }

    public function getStateLicensedIn()
    {
        return $this->settings['stateLicensedIn'] ?? null;
    }


    /**
     * Sets Area Bias option to Postal City
     *
     * @since 2.6.0
     * @see docs/Area Bias.md
     *
     * @return $this
     */
    public function setPostalCityBias()
    {
        $postalCityBias = 'postal-city';
        $this->overwriteAreaBias($postalCityBias);
        return $this;
    }


    /**
     * Overrides area bias in the options
     *
     * @param $bias string  Area Bias
     *
     * @return $this
     */
    protected function overwriteAreaBias($bias)
    {
        if ($this->areaBias !== $bias)
        {
            $this->settings['areaBias'] = $bias;
            $this->setOptions($this->settings);
            $this->loadOptions();
        }
        return $this;
    }


    /**
     * Returns IRE CRM options
     *
     * Currently we only have the URL.
     *
     * @return array|bool CRM options array or false
     */
    protected function getCRMOptions()
    {
        $crmOptions = get_option('ire_crm');

        return (isset($crmOptions['url']) && !empty($crmOptions['url'])) ? $crmOptions : null;
    }

    /**
     * @return null|array
     */
    public function getGfMappingOptions($force = false)
    {
        if ($this->gfMappingOptions === null || $force === true) {
            $this->gfMappingOptions = get_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, null);
        }

        return $this->gfMappingOptions;
    }


    protected function getGoogleApiSettings()
    {
        return new GoogleApiSettings();
    }


    public static function deleteOptions()
    {
        delete_option(self::KEY);
        delete_option('ire_crm');
    }

    public function getCustomer()
    {
        return $this->customer;
    }

    public function updateCustomer(array $newCustomerInfo) {
        if (!isset($this->settings['customer'])) {
            return;
        }

        $this->settings['customer'] = $newCustomerInfo;

        $this->setOptions($this->settings);
    }

    /**
     * @param array $globalListingOptions
     * @return $this
     */
    public function setGlobalListingOptions(array $globalListingOptions)
    {
        $this->settings['listingStatuses'] = $globalListingOptions;
        $this->setOptions($this->settings);
        $this->loadOptions();

        return $this;
    }

    /**
     * @return null|array
     */
    public function getListingFilteringOptions()
    {

        return $this->settings['listingStatuses'] ?? $this->getDefaultListingsFilteringOptions();
    }

    /**
     * @param array $array
     * @param mixed $value
     *
     * @return array
     */
    private function fillArrayItems(array $array, $value = 0): array
    {
        foreach ($array as $key => $item) {
            $array[$key] = $value;
        }

        return $array;
    }

    private function getDefaultListingStatuses(string $market, array $listingTypes): array
    {
        $statusesByTypes = MarketMetadata::getInstance($market)->getStatuses();
        $resultStatuses = [];
        if (is_array($listingTypes) && count($listingTypes) > 0) {
            foreach ($statusesByTypes as $statuses) {
                if (!is_array($statuses)) {
                    continue;
                }

                $resultStatuses = array_merge($resultStatuses, $statuses);
            }
        }

        $defaultValues = [];
        foreach ($resultStatuses as $key => $status) {
            if (strpos(strtolower($key), 'pending') === false) {
                $defaultValues[$key] = '1';
            } else {
                $defaultValues[$key] = '0';
            }
        }
        return $defaultValues;
    }

    private function getDefaultListingPropertyTypes($market, array $listingsTypes)
    {
        $propertyTypes = [];

        foreach ($listingsTypes as $key => $value) {
            $propTypes = MarketMetadata::getInstance($market)->_getPropertyTypesByListingType($key);
            if ($propertyTypes !== false && is_array($propTypes)) {
                $propertyTypes[$key] = $this
                    ->fillArrayItems($propTypes, '1')
                ;
            } else {
                $propertyTypes[$key] = [];
            }

        }

        return $propertyTypes;
    }

    private function getDefaultListingsFilteringOptions(): array
    {
        $markets = Membership::getInstance()->getMarkets();
        $resultListingTypes = [];
        $listingStatusesMarkets = [];
        if (is_array($markets) && count($markets) > 0) {

            foreach ($markets as $market) {
                $listingTypes = MarketMetadata::getInstance($market)->getListingTypes();
                $resultListingTypes = array_merge($resultListingTypes, $listingTypes);
                $listingStatusesMarkets[$market]['listing-types'] = $this->fillArrayItems($listingTypes, '1');
                $listingStatusesMarkets[$market]['listing-statuses'] = $this
                    ->getDefaultListingStatuses($market, $listingStatusesMarkets[$market]['listing-types'])
                ;
                $listingStatusesMarkets[$market]['property-types'] = $this
                    ->getDefaultListingPropertyTypes($market, $listingStatusesMarkets[$market]['listing-types'])
                ;
            }
        }

        $globalListingTypes = $this->fillArrayItems($resultListingTypes, '1');
        foreach ($globalListingTypes as $key => $value) {
            $globalListingTypes[$key] = '1';
        }

        return [
            'global-listing-types' => $globalListingTypes,
            'markets' => $listingStatusesMarkets,
        ];
    }

    public function getListingStatuses($market)
    {
        $listingStatuses = $this->settings['listingStatuses']['markets'][$market]['listing-statuses'] ?? null;

        $result = [];
        if (is_array($listingStatuses) && count($listingStatuses) > 0) {
            foreach ($listingStatuses as $key => $value) {
                if ($value === '1') {
                    $result[] = $key;
                }
            }
        }

        return $result;
    }

    public function getPropertyTypesByMarketAndListingType($market, $listingType)
    {
        $propertyTypes = $this->settings['listingStatuses']['markets'][$market]['property-types'][$listingType] ?? [];

        $result = [];
        if (is_array($propertyTypes) && count($propertyTypes) > 0) {
            foreach ($propertyTypes as $key => $value) {
                if (!empty($value)) {
                    $result[] = $key;
                }
            }
        }

        return $result;
    }
}
