<?php

namespace hji\membership\models;

use hji\common\models\SessionDataCache;

/**
 * Class UserSession
 *
 * @since 2.0
 * @package hji\membership\models
 */

class UserSession
{
    private static $_instance;

    protected $token = null;

    protected $tokenExpires = null;

    protected $user = null;

    protected $vars = array();

    private $dataCache;

    private function __construct()
    {
        $this->dataCache = new SessionDataCache;

        /**
         * IMPORTANT: do not remove!
         * When session already exists, we only call getToken() to check if it's valid
         * but instance properties get never populated from the $_SESSION
         * When calling setUser() => _updateSession() we overwrite $_SESSION with null properties
         * form the current class instance. That's why instance property values need to be
         * pre-populated from $_SESSION inside the constructor.
         */
        $this->token        = self::getToken();
        $this->tokenExpires = self::getTokenExpires();
        $this->user         = self::getUser();
        $this->vars         = self::getVars();
    }


    public static function getInstance()
    {
        if (!( self::$_instance instanceof self))
        {
            self::$_instance = new self();
        }

        return self::$_instance;
    }


    public static function getSessionId()
    {
        return 'hjiUserSession_BlogId_' . get_current_blog_id();
    }


    /**
     * Sets user session with API token and token expiration
     *
     * @param $token
     * @param $tokenExpires
     */
    public static function start($token, $tokenExpires)
    {
        // delete existing session
        self::_deleteSession();

        $instance = self::getInstance();

        $instance->token        = $token;
        $instance->tokenExpires = $tokenExpires;
        $instance->user         = null;

        self::_updateSession();
    }


    public static function set($name, $val)
    {
        $instance = self::getInstance();

        $instance->dataCache->increment('requested_set');

        $instance->vars[$name] = $val;
        self::_updateSession();
    }


    public static function get($name)
    {
        $instance = self::getInstance();

        if (isset($instance->vars[$name])) {
            $instance->dataCache->increment('requested_get');
            return $instance->vars[$name];
        }
        $instance->dataCache->increment('requested_failed');
        $instance->dataCache->arrayPush('requested_failed_vars', $name);
    }

    public static function getVars()
    {
        $dataCache = new SessionDataCache;
        $vars = $dataCache->getData(self::getSessionId() . '_vars');
        if (self::getToken() && !empty($vars)) {
            return $vars;
        }

        return [];
    }


    /**
     * Returns user API token or false if it's not set or expired
     *
     * @return mixed
     */
    public static function getToken()
    {
        $dataCache = new SessionDataCache;
        $token = $dataCache->getData(self::getSessionId() . '_token');
        $expires = $dataCache->getData(self::getSessionId() . '_expires');

        if ($token && $expires && $expires > time()) {
            return $token;
        } else {
            self::_deleteSession();
        }
    }

    /**
     * Returns user API token expiration timestamp or false if token is expired or not set
     *
     * @return mixed
     */
    public static function getTokenExpires()
    {
        $dataCache = new SessionDataCache;
        if (self::getToken()) {
            return $dataCache->getData(self::getSessionId() . '_expires');
        }
    }


    /**
     * Saves user data in a session
     *
     * @param $user
     */
    public static function setUser($user)
    {
        $instance = self::getInstance();
        $instance->user = $user;

        self::_updateSession();
    }


    /**
     * Returns user data from a session
     *
     * @return mixed
     */
    public static function getUser()
    {
        $dataCache = new SessionDataCache;
        if (self::getToken()) {
            return $dataCache->getData(self::getSessionId() . '_user');
        }
    }


    public static function destroy()
    {
        self::_deleteSession();
    }


    /**
     * Updates user session with values of the class instance
     */
    private static function _updateSession()
    {
        $instance = self::getInstance();
        $instance->dataCache->setData(self::getSessionId() . '_token', $instance->token);
        $instance->dataCache->setData(self::getSessionId() . '_expires', $instance->tokenExpires);
        $instance->dataCache->setData(self::getSessionId() . '_user', $instance->user);
        $instance->dataCache->setData(self::getSessionId() . '_vars', $instance->vars);
    }


    /**
     * Deletes user session
     */
    private static function _deleteSession()
    {
        $dataCache = new SessionDataCache;
        $dataCache->unsetData([
            self::getSessionId() . '_token',
            self::getSessionId() . '_tokenExpires',
            self::getSessionId() . '_user',
            self::getSessionId() . '_vars',
            self::getSessionId() . '_expires',
        ]);
    }
}
