<?php

namespace hji\membership\utils;

use hji\membership\Membership;
use Timber\ImageHelper;
use Timber\Twig;

class Image
{
    /**
     * @param $url
     * @param null $width
     * @return mixed|string
     */
    public static function resizeSelfHostedImage(string $url, $width = null, $crop = 'default')
    {
        $parsedUrl = parse_url($url);
        if (!$parsedUrl) {

            return $url;
        }

        if ($width === null) {
            $width = '450';
        }

        if (isset($parsedUrl['host']) && $parsedUrl['host'] === 'listing-images.homejunction.com') {
            if (isset($parsedUrl['query'])) {
                $parsedUrl['query'] .= '&width=' . $width;
            } else {
                $parsedUrl['query'] = 'width=' . $width;
            }

            return URLHelper::unparseUrl($parsedUrl);
        }

        return ImageHelper::resize($url, $width, 0, $crop);
    }

    /**
     * @param string $url
     * @param string $crop
     * @param string $classes
     * @param array $attributes
     * @return string
     */
    private static function getLocalPicture($url, string $crop = 'default', string $classes = '', array $attributes = []): string
    {
        $croppedImgUrl = ImageHelper::resize($url, 450, 0, $crop);

        return '<picture>
                        <source data-srcset="' . ImageHelper::img_to_webp($croppedImgUrl) . '">
                        ' . self::getImage($url, 450, $classes, $attributes) . '
                    </picture>';
    }

    /**
     * @param string $url
     * @param $classes
     * @param $attributes
     * @param $gridSize
     * @param $crop
     * @return string
     */
    public static function getPicture($url, $classes = '', $attributes = [], $gridSize, $crop = 'default')
    {
        if (empty($url)) {
            return '';
        }

        if (URL::isLocalUrl($url)) {
            return self::getLocalPicture($url, $crop, $classes, $attributes);
        }

        $hjiListingHeroImgOptions = Membership::getInstance()->getHjiListingImagesThemeConfig();

        if (
            empty($hjiListingHeroImgOptions)
            || (is_array($hjiListingHeroImgOptions) && count($hjiListingHeroImgOptions[0]) === 0)
        ) {
            return self::getImage($url, 0, true, $classes, $attributes);
        }

        $classesArray = !empty($classes) ? explode(' ', $classes) : [];
        $dataAtr = true;
        if (is_array($classesArray) && count($classesArray) > 0 && in_array('lazzy', $classesArray, true)) {
            $dataAtr = false;
        }

        $picture = '<picture>%s</picture>';

        $sourceHtml = self::getSorceHtml($hjiListingHeroImgOptions[0], $url, $gridSize,$crop, $dataAtr);

        return sprintf($picture, $sourceHtml . self::getImage($url, 0, $classes, $attributes));
    }

    /**
     * @param string $url
     * @param $width
     * @param $classes
     * @param $attributes
     * @return string
     */
    private static function getImage(string $url, $width, $classes = '', $attributes = []):string
    {
        $img = '<img data-src="%s" class="%s" %s >';

        $attributesHtml = '';
        foreach ($attributes as $key => $value) {
            $attributesHtml .= $key . '="' . $value . '" ';
        }

        if (!empty($width)) {
            if (URL::isLocalUrl($url)) {
                $url = self::resizeSelfHostedImage($url, $width);
            } else {
                $url .= '?width=' . $width;
            }
        }

        return sprintf($img, $url, $classes, $attributesHtml);
    }

    /**
     * @param $post
     * @param array $hjiListingHeroImgOptions
     * @return string
     */
    public static function getImagesPreload($post, array $hjiListingHeroImgOptions): string
    {
        $html = '';

        if (!empty($_COOKIE['hji-ridx-img-width']) && !empty($_COOKIE['hji-ridx-viewport-width'])) {
            return '';
        }

        if (isset($post->images)) {

            /**@var \hji\membership\models\data\Image $img*/
            $img = $post->images->getPrimary();

            if ($img !== null) {
                $imgs = $post->images->getAll();

                if (is_array($hjiListingHeroImgOptions)) {
                    $html .= self::getPreloadLinksHtml($hjiListingHeroImgOptions[0], $imgs);
                } else {
                    $html .= '<link rel="preload" as="image" href="' . $img->__get('url') . '?width=320" fetchpriority="high">';
                }
            } else {
                $imgUrl = $post->no_img;

                if (is_array($hjiListingHeroImgOptions)) {
                    $html .= self::getPreloadLinksHtml($hjiListingHeroImgOptions[0], $imgUrl, 'image/jpeg');
                } else {
                    $html .= '<link rel="preload" as="image" href="' . self::resizeSelfHostedImage($imgUrl, 1200) . '">';
                }
            }
        }

        return $html;
    }

    /**
     * @param array $option
     * @param array $images
     * @param $maxViewportSize
     * @param $gridSize
     * @param string $mimeType
     * @param string $media
     * @return array
     */
    private static function getTagPreloadLink(array$option,array $images, $maxViewportSize, $gridSize, string $mimeType, string $media)
    {
        $def_link = '<link rel="preload" as="image" href="%s" fetchpriority="high" media="%s" type="%s">';
        $links = [];

        if (!is_array($option)) {
            return $links;
        }

        $cookieImgWidth = null;
        $cookieWindowWidth = null;
        if (!empty($_COOKIE['hji-ridx-img-width'])) {
            $cookieImgWidth = $_COOKIE['hji-ridx-img-width'];
        }

        if (!empty($_COOKIE['hji-ridx-viewport-width'])) {
            $cookieWindowWidth = $_COOKIE['hji-ridx-viewport-width'];
        }

        if (!empty($option['images']) && is_array($option['images'])) {
            foreach ($option['images'] as $key => $item) {
                if (isset($images[$key])) {
                    $url = $images[$key]->__get('url');
                    $isLocal = URL::isLocalUrl($url);

                    if (is_array($item)) {
                        foreach ($item as $sizeKey => $values) {
                            if (
                                $key === 0
                                && $sizeKey === 0
                                && $cookieImgWidth !== null
                                && $cookieWindowWidth !== null
                                && $cookieImgWidth >= ($option['screen']['min'] ?? 0)
                                && $cookieWindowWidth <= $option['screen']['max']
                            ) {
                                $size = $cookieImgWidth;
                            } elseif (!empty($values['viewport-width'])) {
                                $size = $values['viewport-width'];
                            } else {
                                $size = $maxViewportSize;
                            }

                            if ($size !== null) {
                                $columns = empty($values['columns']) ? 1 : $values['columns'];
                                $gap = empty($values['gap']) ? 0 : $values['gap'];
                                $koef = empty($values['koef']) ? 1 : $values['koef'];

                                if (!empty($gridSize) && $columns === 'grid_size') {
                                    $columns = $gridSize;
                                } elseif ($columns === 'grid_size') {
                                    $columns = 3;
                                }

                                $imageSize = ceil(($size - $gap) * $koef / $columns);

                                if ($isLocal) {
                                    $imgUrl = self::resizeSelfHostedImage($url, $imageSize);
                                } else {
                                    $imgUrl = $url . '?width=' . $imageSize;
                                }
                            } else {
                                $imgUrl = $url;
                            }

                            $k = (string) $size;
                            if (isset($links[$k])) {
                                $links[$k] .= sprintf($def_link, $imgUrl, $media, $mimeType);
                            } else {
                                $links[$k] = sprintf($def_link, $imgUrl, $media, $mimeType);
                            }
                        }
                    } elseif ($item === null) {
                        if (isset($links[$key])) {
                            $links[(string)$key] .= sprintf($def_link, $url, $media, $mimeType);
                        } else {
                            $links[(string)$key] = sprintf($def_link, $url, $media, $mimeType);
                        }
                    }

                    }
            }
        }

        return $links;
    }

    private static function getPreloadLinksHtml($options, $images, $mimeType = 'image/jpeg')
    {
        $links = [];

        foreach ($options as $option) {
            if (isset($option['screen']) && count($option['screen']) === 2 && isset($option['screen']['min']) && isset($option['screen']['max'])) {
                $media = "(min-width: {$option['screen']['min']}px) and (max-width: {$option['screen']['max']}px)";

                $links = array_merge($links, self::getTagPreloadLink($option, $images, $option['screen']['max'], 3, $mimeType, $media));

            } elseif (isset($option['screen']) && count($option['screen']) === 1 && (isset($option['screen']['min']) || isset($option['screen']['max']))) {
                if (!empty($option['screen']['max'])) {
                    $media = "(max-width:{$option['screen']['max']}px)";
                } elseif (!empty($option['screen']['min'])) {
                    $media = "(min-width:{$option['screen']['min']}px)";
                }

                $maxViewportSize = empty($option['screen']['max']) ? null : $option['screen']['max'];
                $links = array_merge($links, self::getTagPreloadLink($option, $images, $maxViewportSize, 3, $mimeType, $media));
            } elseif ((isset($option['screen']) && count($option['screen']) === 0) || !isset($option['screen'])) {
                $def_link2 = '<link rel="preload" as="image" href="%s" fetchpriority="high" media="all" type="%s">';
                $maxViewportSize = empty($option['screen']['max']) ? null : $option['screen']['max'];

                if (!empty($option['images'] && is_array($option['images']))) {
                    foreach ($option['images'] as $key => $item) {
                        if (isset($images[$key])) {
                            $url = $images[$key]->__get('url');

                            foreach ($item as $values) {
                                if (!empty($values['viewport-width'])) {
                                    $size = $values['viewport-width'];
                                } else {
                                    $size = $maxViewportSize;
                                }

                                if ($size !== null) {
                                    $imgUrl = $url . '?width=' . $size;
                                } else {
                                    $imgUrl = $url;
                                }

                                $k = (string) $size;
                                if (isset($links[$k])) {
                                    $links[$k] .= sprintf($def_link2, $imgUrl, $mimeType);
                                } else {
                                    $links[$k] = sprintf($def_link2, $imgUrl, $mimeType);
                                }
                            }
                        }
                    }
                }
            }
        }

        return implode('', $links);
    }

    /**
     * @param array $option
     * @param string $url
     * @param int $maxViewportSize
     * @param string $media
     * @param string $crop
     * @return array|string
     */
    public static function getTagSource(array $option, string $url, int $maxViewportSize, string $media, $gridSize, string $crop, $dataAtr)
    {
        if (!is_array($option)) {
            return [];
        }

        $dataSrcrSet = $dataAtr ? 'data-' : '';
        $defLink = '<source %ssrcset="%s" fetchpriority="high" media="%s" />';
        $isLocal = URL::isLocalUrl($url);

        if (is_array($option) && count($option) === 0) {
            if ($isLocal) {
                $imgUrl = self::resizeSelfHostedImage($url, $maxViewportSize);
            } else {
                $imgUrl = $url . '?width=' . $maxViewportSize;
            }

            $links = sprintf($defLink, $dataSrcrSet, $imgUrl, $media);

            if ($isLocal) {
                $links .= sprintf($defLink, $dataSrcrSet, ImageHelper::img_to_webp($imgUrl, 100), $media);
            }

            return $links;
        }

        $links = [];

        $cookieImgWidth = null;
        $cookieWindowWidth = null;
        if (!empty($_COOKIE['hji-ridx-img-width'])) {
            $cookieImgWidth = $_COOKIE['hji-ridx-img-width'];
        }

        if (!empty($_COOKIE['hji-ridx-viewport-width'])) {
            $cookieWindowWidth = $_COOKIE['hji-ridx-viewport-width'];
        }

        if (!empty($option['images']) && is_array($option['images'])) {
            foreach ($option['images'] as $key => $item) {
                if (is_array($item)) {
                    foreach ($item as $sizeKey => $values) {
                        if (
                            $key === 0
                            && $sizeKey === 0
                            && $cookieImgWidth !== null
                            && $cookieWindowWidth !== null
                            && $cookieImgWidth >= ($option['screen']['min'] ?? 0)
                            && $cookieWindowWidth <= $option['screen']['max']
                        ) {
                            $size = $cookieImgWidth;
                        } elseif (!empty($values['viewport-width'])) {
                            $size = $values['viewport-width'];
                        } else {
                            $size = $maxViewportSize;
                        }

                        if (!empty($size)) {
                            $columns = empty($values['columns']) ? 1 : $values['columns'];
                            $gap = empty($values['gap']) ? 0 : $values['gap'];
                            $koef = empty($values['koef']) ? 1 : $values['koef'];
                            $gapCard = empty($values['gap_card']) ? 0 : $values['gap_card'];

                            if (!empty($gridSize) && $columns === 'grid_size') {
                                $columns = $gridSize;
                            } elseif ($columns === 'grid_size') {
                                $columns = 3;
                            }

                            $imageSize = ceil(($size - $gap - $gapCard * $columns) * $koef / $columns);

                            if ($isLocal) {
                                $imgUrl = ImageHelper::resize($url, $imageSize, 0, $crop);
                            } else {
                                $imgUrl = $url . '?width=' . (int)$imageSize;
                            }
                        } else {
                            $imgUrl = $url;
                        }

                        $k = (string) $size;

                        if (isset($links[$k])) {
                            $links[$k] .= sprintf($defLink, $dataSrcrSet, $imgUrl, $media);
                        } else {
                            $links[$k] = sprintf($defLink, $dataSrcrSet, $imgUrl, $media);
                        }

                        if ($isLocal) {
                            $links[$k] .= sprintf($defLink, $dataSrcrSet, ImageHelper::img_to_webp($imgUrl, 100), $media);
                        }
                    }
                } elseif ($item === null) {
                    if (isset($links[$key])) {
                        $links[(string)$key] .= sprintf($defLink, $dataSrcrSet, $url, $media);
                    } else {
                        $links[(string)$key] = sprintf($defLink, $dataSrcrSet, $url, $media);
                    }
                }
            }
        }

        return $links;
    }

    private static function getSorceHtml($options, $images, $gridSize, $crop, $dataAtr)
    {
        $links = [];

        foreach ($options as $option) {
            if (isset($option['screen']) && count($option['screen']) === 2 && isset($option['screen']['min']) && isset($option['screen']['max'])) {
                $media = "(min-width: {$option['screen']['min']}px) and (max-width: {$option['screen']['max']}px)";

                $links = array_merge($links, self::getTagSource($option, $images, $option['screen']['max'], $media, $gridSize, $crop, $dataAtr));

            } elseif (isset($option['screen']) && count($option['screen']) === 1 && (isset($option['screen']['min']) || isset($option['screen']['max']))) {
                if (!empty($option['screen']['max'])) {
                    $media = "(max-width:{$option['screen']['max']}px)";
                } elseif (!empty($option['screen']['min'])) {
                    $media = "(min-width:{$option['screen']['min']}px)";
                }

                $maxViewportSize = empty($option['screen']['max']) ? 0 : $option['screen']['max'];
                $links = array_merge($links, self::getTagSource($option, $images, $maxViewportSize, $media, $gridSize, $crop, $dataAtr));
            } elseif ((isset($option['screen']) && count($option['screen']) === 0) || !isset($option['screen'])) {
                $def_link2 = '<link rel="preload" as="image" href="%s" fetchpriority="high" media="all">';
                $maxViewportSize = empty($option['screen']['max']) ? 0 : $option['screen']['max'];

                if (!empty($option['images'] && is_array($option['images']))) {
                    foreach ($option['images'] as $key => $item) {
                        if (isset($images[$key])) {
                            $url = $images[$key]->__get('url');

                            foreach ($item as $values) {
                                if (!empty($values['viewport-width'])) {
                                    $size = $values['viewport-width'];
                                } else {
                                    $size = $maxViewportSize;
                                }

                                if ($size !== null) {
                                    $imgUrl = $url . '?width=' . $size;
                                } else {
                                    $imgUrl = ImageHelper::resize($url, $size, 0, $crop);
                                }

                                $k = (string) $size;
                                if (isset($links[$k])) {
                                    $links[$k] .= sprintf($def_link2, $imgUrl);
                                } else {
                                    $links[$k] = sprintf($def_link2, $imgUrl);
                                }
                            }
                        }
                    }
                }
            }
        }

        return implode('', $links);
    }

    public static function getLocalImageWidth($url)
    {
        if ($url instanceof \Timber\Image) {
            return $url->width();
        }

        $imageId = attachment_url_to_postid($url);
        if ($imageId === 0) {
            return 0;
        }

        $imageMeta = wp_get_attachment_metadata($imageId);

        if (!empty($imageMeta['width'])) {
            return $imageMeta['width'];
        }

        return 0;
    }

    public static function getImagesByPost($post)
    {

    }
}
