<?php

namespace hji\common\interfaces;

use hji\common\utils\StringUtil;
use hji\membership\Membership;
use Timber\Post;

abstract class PostContract extends Post
{
    public $schema;
    public $should_crop = true;
    public $no_img;

    public function __construct($pid = null, $no_img = null)
    {
        parent::__construct($pid);
        $this->setSchema();

        if ($no_img === null && isset(Membership::$noImgUrl)) {
            $this->no_img = Membership::$noImgUrl;
        } else {
            $this->no_img = $no_img;
        }

    }


    /**
     * Builds the post class
     *
     * @param string $primary_class
     * @param array  $additional_classes
     * @return string Class names for the post template
     */
    public function post_class($primary_class = 'hjitem', $additional_classes = array())
    {
        $primary_class = strtolower($primary_class);
        $classes = array($primary_class, $primary_class . '--' . $this->post_type);

        if (($this->post_type != 'page') && ($this->object_type != $this->post_type))
        {
            $classes[] = $primary_class . '--' . $this->object_type;
        }

        // Post Format
        if (post_type_supports($this->post_type, 'post-formats'))
        {
            $post_format = get_post_format($this->ID);

            if ($post_format && !is_wp_error($post_format))
            {
                $classes[] = 'format--' . sanitize_html_class($post_format);
            }
            else
            {
                $classes[] = 'format--standard';
            }
        }

        // Post thumbnails.
        if (has_post_thumbnail($this->ID) && !is_attachment($this))
        {
            $classes[] = 'post--has-thumbnail';
        }

        // sticky for Sticky Posts
        if (is_sticky($this->ID) && is_home() && !is_paged())
        {
            $classes[] = 'sticky';
        }

        // hentry for hAtom compliance
        if ($this->post_type === 'post')
        {
            $classes[] = 'hentry';
        }

        // All public taxonomies
        $taxonomies = get_taxonomies(array('public' => true));
        foreach ((array)$taxonomies as $taxonomy)
        {
            if (is_object_in_taxonomy($this->post_type, $taxonomy))
            {
                foreach ((array)get_the_terms($this->ID, $taxonomy) as $term)
                {
                    if (empty($term->slug))
                    {
                        continue;
                    }

                    $term_class = sanitize_html_class($term->slug, $term->term_id);
                    if (is_numeric($term_class) || !trim($term_class, '-'))
                    {
                        $term_class = $term->term_id;
                    }

                    $classes[] = sanitize_html_class($taxonomy . '--' . $term_class, $taxonomy . '--' . $term->term_id);
                }
            }
        }

        // Adds custom css classes from a theme
        if (is_array($additional_classes))
        {
            $classes = array_merge($classes, $additional_classes);
        }

        /**
         * Allows themes to override / append classes on posts using a filter
         * @TODO: decide if we really want this or if the array_merge above is enough
         */
        //$classes = apply_filters('blvd/post/classes', $classes);

        return join(' ', $classes);
    }


    /**
     * Cleans up object vars by removing an namespaced fields.
     * For example, pocket listings meta-data fields all start with '_pl_' and we don't want that
     * when used in our templates because it hinders re-usability.
     *
     * @since 1.0.0
     * @param $field_namespace
     */
    protected function _cleanUpObject($field_namespace)
    {
        $vars = get_object_vars($this);

        foreach ($vars as $var => $value)
        {
            if (mb_strpos($var, $field_namespace) !== false)
            {
                unset($this->{$var});
            }
        }
    }


    /**
     * Maps data from one object this (post) object
     *
     * @since 1.0.0
     * @param object $obj A data object
     */
    protected function _mapDataObj($obj)
    {
        if (!is_object($obj)) {
            throw new \Exception('_mapDataObj only accepts an object data type');
        }
        // Get data from slipstream listing and map to this obj
        $data = get_object_vars($obj);
        foreach ($data as $key => $value)
        {
            // if we've already mapped a field we don't want to override it
            // For example, images are now using the Image->Images Objects created for New Construction and we don't
            // want to override that field after we've already set it up before mapping the remaining data.
            if (!isset($this->{$key}))
            {
                $this->{$key} = $value;
            }
        }
    }


    /**
     * Maps all custom namespaced fields to a non-namespaced attribute
     * Any fields that need to be skipped should be assigned before calling
     * this method so that they are not lost because we unset them either way.
     *
     * @since 1.0.0
     * @param string $field_namespace field namespace - start of string only
     * @param array  $skipped_fields any fields which shouldn't be remapped
     */
    protected function _mapFields($field_namespace, $skipped_fields = array())
    {
        $vars = get_object_vars($this);

        foreach ($vars as $var => $value)
        {
            if (mb_strpos($var, $field_namespace) !== false && !in_array($var, $skipped_fields))
            {
                $clean_field        = str_replace($field_namespace, '', $var);
                $this->$clean_field = $value;
            }
        }
    }

    /**
     * Check current single post permalink versus parent or sibling permalink
     *
     * @since 1.0.0
     * @param string
     * @return boolean
     */
    protected function _isCurrentSinglePage()
    {
        if (get_permalink() == $this->link() && is_singular($this->post_type))
        {
           return true;
        }
        return false;
    }


    private function setSchema()
    {
        if (!$this->schema)
        {
            $this->schema = 'WebPageElement';
            $this->data_attributes = $this->data_attributes . ' itemprop="mainContentOfPage"';
        }
    }

    /**
     * Override Timber Post get_field
     * remove get_revised_data_from_method, which create infinity loop
     * @param string $field_name
     * @return array|mixed|\Timber\mix|void|\WP_Post|null
     */
    public function get_field($field_name)
    {
//        if ( $rd = $this->get_revised_data_from_method('get_field', $field_name) ) {
//            return $rd;
//        }
        $value = apply_filters('timber_post_get_meta_field_pre', null, $this->ID, $field_name, $this);
        if ($value === null) {
            $value = get_post_meta($this->ID, $field_name);
            if (is_array($value) && count($value) == 1) {
                $value = $value[0];
            }
            if (is_array($value) && count($value) == 0) {
                $value = null;
            }
        }
        $value = apply_filters('timber_post_get_meta_field', $value, $this->ID, $field_name, $this);
        $value = $this->convert($value);

        return $value;
    }

    /**
     * @param $text
     * @return string|string[]|null
     */
    protected function fullBathBedRoom($text)
    {
        return StringUtil::fullBathBedRoom($text);
    }

}
