<?php

namespace hji\common\models;

use hji\common\utils\ShortcodeBinding;
use hji\membership\Membership;
use Timber\Timber;
use Timber\Image;


trait ListingTrait
{
    /**
     * @param $agent_id
     * @param $market
     * @return mixed|null
     */
    protected function _getAgent($agent_id, $market = null)
    {
        if (!$agent_id) {
            return null;
        }

        $args   = [
            'posts_per_page' => 1,
            'post_type'      => 'agent',

        ];

        if ($market) {
            $args['meta_query']   = [
                [
                    'key'     => '_cmb_agent_id',
                    'value'   => $market . ':' . $agent_id,
                    'compare' => 'LIKE'
                ]
            ];
        } else {
            $args['p']   = $agent_id;
        }

        $result = Timber::get_posts($args, 'hji\AgentRoster\models\Agent');

        if (!$result) {
            return null;
        }

        return $result[0] ?? null;
    }


    /**
     * Gets the Agent Contact Information
     * Shows the agent information on a listing details page.
     * Will fall back to theme options when it can't match an agent.
     *
     * @since 0.13.0
     * @param $agent_id mixed The agent MLS id
     * @param $market   mixed Accepts the market ID as a string
     * @param $agent    mixed Accepts hji\AgentRoster\models\Agent object
     * @return object   Agent contact info for an agent card
     */
    protected function _getAgentAttribution($agent_id = false, $market = false, $agent = null)
    {
        $object = null;

        if ($agent) {
            $object = $agent;
        } elseif ( $agent_id && ($agent = $this->_getAgent($agent_id, $market))) {
            $object = $agent;
        } elseif (function_exists('blvdGetOption')) {
            if (class_exists(\hji\AgentRoster\models\Agent::class)) {
                $object = new \hji\AgentRoster\models\Agent();
            } else {
                $object = new \hji\membership\models\AgentFake();
            }

            $image = blvdGetOption('agent_image', 'id');
            $object->class        = 'hji-post hji-post--agent';
            $object->thumbnail    = new Image($image);
            $object->link         = null;
            $object->title        = blvdGetOption('agent_name');
            $object->job_title    = blvdGetOption('agent_title');
            $object->designation  = $this->_getAgentDesignation();
            $object->email        = blvdGetOption('agent_email_address');
            $object->mobile_phone = blvdGetOption('agent_mobile_phone');
            $object->office_phone = blvdGetOption('agent_office_phone');
            $object->facebook     = blvdGetOption('facebook_link');
            $object->twitter      = blvdGetOption('twitter_link');
            $object->google_plus  = blvdGetOption('google_plus_link');
            $object->linkedin     = blvdGetOption('linkedin_link');
            $object->youtube      = blvdGetOption('youtube_link');
            $object->instagram    = blvdGetOption('instagram_link');
            $object->pinterest    = blvdGetOption('pinterest_link');
            $object->yelp         = blvdGetOption('yelp_link');
            $object->zillow       = blvdGetOption('zillow_link');
            $object->office       = [
                'street'         => blvdGetOption('agent_address', 'street'),
                'city'           => blvdGetOption('agent_address', 'city'),
                'state_province' => blvdGetOption('agent_address', 'state'),
                'postal_code'    => blvdGetOption('agent_address', 'zip')
            ];

            $imgPath = $object->thumbnail->file_loc;
            if (!file_exists($imgPath)) {
                $object->thumbnail = new Image(Membership::$noAgentImgUrl);
            }

            if (isset($object->facebook, $object->twitter, $object->google_plus, $object->linkedin, $object->pinterest, $object->youtube, $object->instagram, $object->yelp)) {
                $object->has_social = true;
            }
            }

        return $object;
    }


    /**
     * Get Agent designation based upon state regulation / laws
     *
     * @TODO: decide if we need to change this...maybe a public function that can be used anywhere...just pass a state.
     * @since 0.13.0
     * @return string
     */
    protected function _getAgentDesignation()
    {
        $customer = Membership::getInstance()->customerModel->customer;
        $agent_state = 'CA'; //Hardcoded

        if (isset($customer->address)) {
            $agent_state = $customer->address->state;
        }

        return $agent_state . ($agent_state == 'CA' ? ' BRE' : ' DRE');
    }


    /**
     * Check for listing lat and lng
     *
     * @since 0.13.0
     * @return bool
     */
    protected function _hasLocation()
    {
        return is_object($this->coordinates) || is_array($this->coordinates);
    }


    /**
     * Creates standardized coordinates object
     *
     * @since 0.13.0
     * @param $lat float Latitude value
     * @param $lng float Longitude value
     * @return \stdClass Coordinates object
     */
    protected function _getCoordinates($lat, $lng)
    {
        $coordinates = new \stdClass();
        $coordinates->latitude = (float) $lat;
        $coordinates->longitude = (float) $lng;

        return $coordinates;
    }


    /**
     * Returns nearby schools slipstream module
     *
     * @since 0.13.0
     * @param string $type coords or polygon are the only supported types
     * @param string $polygon polygon ID w/o dollar sign prefix
     * @return bool|string
     */
    protected function _getNearbySchools($type = 'coords', $polygon = null)
    {
        if (is_plugin_active('hji-schools/schools.php') && $this->_hasLocation())
        {
            $output = null;

            if ($type == 'coords')
            {
                $output = (new ShortcodeBinding($this->object_type.'/schools'))->setShortcodeID('schools_nearby');
                $output->coords("{$this->coordinates->latitude}:{$this->coordinates->longitude}");
            }
            elseif ($type == 'polygon' && $polygon)
            {
                $output = (new ShortcodeBinding($this->object_type.'/schools'))->setShortcodeID('schools');
                $output->polygon('$' . $polygon);
            }

            return $output;
        }

        return false;
    }


    /**
     * Returns nearby sales slipstream module
     *
     * @since 0.13.0
     * @return bool|string
     */
    protected function _getNearbySales()
    {
        if (is_plugin_active('hji-recent-sales/recent-sales.php')
            && $this->_hasLocation()
            && self::allowsRecentSales($this->market)
        )
        {
            return (new ShortcodeBinding($this->object_type.'/recent_sales'))
                ->setShortcodeID('recent_sales')
                ->days(180)
                ->sortfield('saledate')
                ->sortorder('desc')
                ->hide_search_form(1)
                ->circle("{$this->coordinates->latitude},{$this->coordinates->longitude}:15");
        }
        else
        {
            return false;
        }
    }


    protected function _getStatus($field)
    {
        return array(
            'name'  => 'listing_status',
            'label' => 'Status',
            'value' => $field
        );
    }


    /**
     * Gets the correct virtual tour URL for the listing
     *
     * @since 1.0.0
     * @return bool
     */
    private function _getTourUrl()
    {
        $possible_attrs = array(
            'tourURL',
            'tourUrl',
            '_pl_video_code'
        );

        foreach ($possible_attrs as $attr)
        {
            if (property_exists($this, $attr) && $this->$attr != '')
            {
                return $this->$attr;
            }
        }

        return false;
    }


    /**
     * Filters the status on listings secondary field maps
     * @return mixed|void
     */
    private function _removeStatusFromSecondaryFields()
    {
        return apply_filters('blvd/listing/status', false);
    }


    /**
     * Tells us if a market does not allow recent sales
     *
     * @since 0.11.0
     * @param $market
     * @return bool
     */
    public static function allowsRecentSales($market)
    {
        $restricted_markets = array(
            'actris',   // TX
            'txccmls',  // TX
            'ctxmls',   // TX
            'tylertx',  // TX
            'hlmls',    // TX
            'harmls',   // TX
            'laar',     // TX
            'lubar',    // TX
            'ntreis',   // TX
            'rpaar',    // TX
            'sabor',    // TX
            'santafe',   // NM
            'imls', //ID
        );

        if (in_array($market, $restricted_markets))
        {
            return false;
        }

        return true;
    }

}
