<?php

namespace hji\common\utils;


use hji\membership\Membership;
use hji\membership\models\webservice\abstracts\WebService;
use hji\membership\utils\Environment;
use hji\membership\utils\SearchString;
use hji\membership\utils\URL;
use hji\ResponsiveIDX\models\MarketMetadata;

class APIClient
{
    protected $site_url = null;
    protected $licenseKey = null;

    protected $cacheTransport = false;

    public $api_base = "https://slipstream.homejunction.com";
    public $api_dev_base = "https://slipstream-test.homejunction.com";
    protected $is_staging = false;

    public $last_error_code = null;
    public $last_error_mess = null;
    public $last_token = null;
    public $last_token_expires = null;
    protected $token_header = 'HJI-Slipstream-Token';
    protected $productName = 'hji-wordpress';
    protected $ch = null;
    protected $debug_log;
    protected $debug_mode = false;
    protected $application_name = null;

    private $statusesForInActiveListings = ['status' => ['closed','sold','leased']];

    private $unsetArgsForInactiveListings = ['listingType','propertyType','address.state'];

    /** @var string     API Version*/
    public static $api_version = "v20180104";

    /**
     * @since 2.6.0
     * @see https://slipstream.homejunction.com/docs/ws/areas#bias
     * @var string      Area Bias: census-place, postal-city, none.
     */
    public static $areaBias = null;

    protected $authData = false;

    protected $error_messages = array(
        '200' => 'Code: 200. Success.',
        '400' => 'Code: 400. Your request is invalid. Typically this is because you are missing a required parameter.',
        '401' => 'Code: 401. Invalid license key.',
        '403' => 'Code: 403. You attempted to access a restricted resource.',
        '404' => 'Code: 404. These aren\'t the droids you\'re looking for. Move along!',
        '500' => 'Code: 500. Internal server error.',
        '503' => 'Code: 503. Service unavailable.'
    );

    // pagination vars
    public $last_count = 0;
    public $page_size = 0;
    public $total_pages = 0;
    public $current_page = 0;

    private $membershipSettingsInstance;


    function __construct(Membership $membership)
    {
        $this->membershipSettingsInstance = $membership->settings();
        $this->licenseKey = $membership->settings()->licenseKey;

        // force dev API base if current site is hosted on a Staging HJ server

        if (Environment::is(Environment::DEV))
        {
            $this->setDeveloperMode(true);
        }

        $this->customerModel = $membership->customerModel;
        $this->authData = isset($this->customerModel->license) ? $this->customerModel->license : false;

        $this->InitCurl();

        register_shutdown_function(array(&$this, 'debug_data'));
    }


    function getApiBaseUrl()
    {
        return $this->api_base;
    }


    function setCacheTransport($object)
    {
        $this->cacheTransport = $object;
    }

    function InitCurl()
    {
        // initialize cURL for use later
        $this->ch = curl_init();

        curl_setopt($this->ch, CURLOPT_HEADER, true);
        curl_setopt($this->ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($this->ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($this->ch, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($this->ch, CURLOPT_TIMEOUT, 30);
    }


    function __destruct()
    {
        // clean cURL up
        if ($this->ch)
            curl_close($this->ch);
    }


    ////////////////////////////////////////////////////////////////////
    //  AUTH FUNCTIONS
    ////////////////////////////////////////////////////////////////////


    function authenticate($force = false, array $params = array())
    {
        if (!$this->licenseKey)
        {
            return;
        }

        $this->InitCurl();

        $default['licenseKey'] = $this->licenseKey;
        $default['product']    = $this->productName;
        $default['ipAddress']  = ipAddress::getIP();
        $default['version']    = self::$api_version;
        $default['customer']   = true;
        $default['markets']    = true;
        $default['site']       = parse_url(home_url(), PHP_URL_HOST);

        $params = array_merge($default, $params);

        $result = $this->makeAPIRequest("GET", "/ws/api/authenticate", $params, array(), $auth = true, $force);

        return $result;
    }



    function isTokenExpired()
    {
        $timestamp = time();

        if ($this->last_token_expires == null || $timestamp > $this->last_token_expires)
        {
            return true;
        }

        return false;
    }


    function setDebugMode($mode = false)
    {
        $this->debug_mode = $mode;

        // enable logging if we're in debug mode

        if ($this->debug_mode === true)
        {
            $this->debug_log = fopen(ABSPATH . "debug.log", 'a');
            curl_setopt($this->ch, CURLOPT_VERBOSE, 1);
            curl_setopt($this->ch, CURLOPT_STDERR, $this->debug_log);
        }
    }


    function setDeveloperMode($enable = false)
    {
        if ($enable)
        {
            $this->api_base   = $this->api_dev_base;
            $this->site_url   = 'hjstaging.com';
            $this->is_staging = true;

            return true;
        }
        else
        {
            return false;
        }
    }

    function getErrors()
    {
        if ($this->last_error_code && $this->last_error_mess)
        {
            return $this->last_error_mess;
        }
        else
        {
            return false;
        }
    }


    ////////////////////////////////////////////////////////////////////
    //  CHAIN SLIPSTREAM CALLS
    ////////////////////////////////////////////////////////////////////

    function call($args, $listingType = null)
    {
        if (is_array($args)) {
            foreach ($args as $key => $callItem) {
                if (isset($callItem['uri']) && $callItem['uri'] === '/ws/listings/search') {
                    if (isset($callItem['parameters']) && is_array($callItem['parameters'])) {
                        $args[$key]['parameters'] = $this->filteringRequestParams($callItem['parameters']);
                    } else {
                        $args[$key]['parameters'] = $this->filteringRequestParams();
                    }
                } else if (isset($callItem['uri']) && $callItem['uri'] === '/ws/listings/inactive/search') {
                    $args[$key]['parameters'] = $this->unsetUnusedArgsInInactiveListings($args[$key]['parameters'], $this->unsetArgsForInactiveListings, $this->statusesForInActiveListings);
                }
            }
        }
        $args = array('request' => json_encode($args));

        $result = $this->makeAPIRequest("POST", "/ws/api/call", $args, array(), $auth = false);

        // Just return the results. Whichever call it will process the results
        return $result;
    }


    ////////////////////////////////////////////////////////////////////
    //  SEARCH/LISTING FUNCTIONS
    ////////////////////////////////////////////////////////////////////

    private function filteringLicensedInStates($args)
    {
        if (!isset($args['address.state'])) {
            $licensedInStates = $this->membershipSettingsInstance->getOption('licensedInStates');
            if (!empty($licensedInStates) && count($licensedInStates) > 0) {
                $args['address.state'] = implode(',', $licensedInStates);
            }
        }

        return $args;
    }

    /**
     * @param array $args
     * @return mixed
     */
    private function filteringByListingStatus($args)
    {
        if (!empty($args['status']) && is_string($args['status'])) {
            $temp = $args['status'];
            $args['status'] = [];
            $args['status'][] = $temp;
        }

        if (empty($args['status']) && isset($args['market'])) {
            $listingStatuses = $this->membershipSettingsInstance->getListingStatuses($args['market']);
            if (!empty($listingStatuses) && count($listingStatuses) > 0) {
                $args['status'] = $listingStatuses;
            } elseif (!empty($args['listingType'])) {
                $listingStatuses = MarketMetadata::getInstance($args['market'])->getStatuses();
                $args['status'] = [];
                if (is_array($listingStatuses) && !empty($listingStatuses[$args['listingType']]) && is_array($listingStatuses[$args['listingType']])) {
                    foreach ($listingStatuses[$args['listingType']] as $key => $status) {
                        $keyTmp = strtolower($key);
                        if (strpos($keyTmp, 'pending') === false) {
                            $args['status'][] = $keyTmp;
                        }
                    }

                    if (count($args['status']) === count($listingStatuses[$args['listingType']])) {
                        $args['status'] = [];
                    }
                } else {
                    $args['status'] = [];
                }
            }
        }

        return $args;
    }

    /**
     * @param array $args
     * @param array|null $urlQueryParams
     * @return array
     */
    private function filteringByPropertyTypes(array $args, $urlQueryParams)
    {
        if (!isset($args['propertyType']) && isset($args['market'], $args['listingType'])) {
            $propertyTypes = $this->membershipSettingsInstance->getPropertyTypesByMarketAndListingType($args['market'], $args['listingType']);

            if (!empty($propertyTypes) && count($propertyTypes) > 0) {
                $args['propertyType'] = $propertyTypes;
            }
        }

        if (!empty($urlQueryParams['propertyType'])) {
            $args['propertyType'] = [$urlQueryParams['propertyType']];
        } elseif (isset($args['listingType'])) {
            $propertyTypeOptions = $this->membershipSettingsInstance->getPropertyTypesByMarketAndListingType($args['market'], $args['listingType']);

            if (isset($args['propertyType']) && is_string($args['propertyType'])) {
                $args['propertyType'] = SearchString::searhStringToArray($args['propertyType']);
            }

            if (isset($args['propertyType']) && is_array($args['propertyType'])) {
                $propertyTypeByShortcode = $args['propertyType'];
            }  else {
                $propertyTypeByShortcode = [];
            }

            if (is_array($propertyTypeOptions)) {
                if (count($propertyTypeOptions) === 0) {
                    $args['propertyType'] = $propertyTypeByShortcode;
                } else {
                    $args['propertyType'] = array_intersect($propertyTypeOptions, $propertyTypeByShortcode);
                }

                if (count($args['propertyType']) === 0) {
                    $args['propertyType'] = $propertyTypeOptions;
                }
            } else {
                $args['propertyType'] = [];
            }
        }

        return $args;
    }

    /**
     * @param $args
     * @return array
     */
    private function filteringRequestParams($args)
    {
        $urlQueryParams = null;
        if (!empty($args['referrer'])) {
            $urlQueryParams =  URL::getQueryParams($args['referrer']);
            unset($args['referrer']);
        }

        $args = $this->filteringLicensedInStates($args);

        $args = $this->filteringByListingStatus($args);

        $args = $this->filteringByPropertyTypes($args, $urlQueryParams);

        return $args;
    }

    /**
     * Returns listings that match search parameters
     *
     * @param $args
     * @param bool $transient
     *
     * @return bool|array
     */
    function searchListings($args, bool $transient = false)
    {
        $args = $this->filteringRequestParams($args);

        $listingsLastUpdate = -1;
        $marketMeta = MarketMetadata::getInstance($args['market'])->getMeta();
        $transientKey = md5("/ws/listings/search" . '_' . json_encode($args));

        if (
            !empty($args['market'])
            && $marketMeta
        ) {
            $listingsLastUpdate = $marketMeta->metadata->listings->lastUpdated;
        }

        if (isset($_GET['hjdebug']) && $_GET['hjdebug'] === '1') {
            delete_transient($transientKey);
        } elseif (!defined('HJI_CACHE_DISABLE') || (defined('HJI_CACHE_DISABLE') && HJI_CACHE_DISABLE === false)) {
            $savedLastUpdate = get_option('hji_listings_lastUpdate', 0);
            $now = (new \DateTime('now'))->modify('-4 hour')->getTimestamp();


            if ($transient === true && ($savedLastUpdate === $listingsLastUpdate || $listingsLastUpdate < $now)) {
                if (($result = get_transient($transientKey)) !== false
                    && isset($result['listings'])
                ) {
                    return $result;
                }
            }
        }


        $result = $this->makeAPIRequest("POST", "/ws/listings/search", $args, [], $auth = false);

        if (!isset($result['listings']))
        {
            return false;
        }

        if ($transient === true) {
            ListingsTransient::setListingsLastUpdate($transientKey, $result, $marketMeta);
        }

        return $result;
    }

    /**
     * @param array $args
     * @param array $unsetArgs
     * @param array $overrideArgs
     * @return array
     */
    private function unsetUnusedArgsInInactiveListings(array $args, array $unsetArgs, array $overrideArgs): array
    {
        if (count($unsetArgs) > 0) {
            foreach ($unsetArgs as $value) {
                if (isset($args[$value])) {
                    unset($args[$value]);
                }
            }
        }

        if (count($overrideArgs) > 0) {
            foreach ($overrideArgs as $key => $value) {
                $args[$key] = $value;
            }
        }

        return $args;
    }

    public function searchInActiveListingsByArea($listingsArgs, $areaArgs, bool $transient = false)
    {
        return $this->_searchListingsByArea($listingsArgs, $areaArgs, $transient, 'inactive');
    }

    public function searchListingsByArea($listingsArgs, $areaArgs, bool $transient = false)
    {
        return $this->_searchListingsByArea($listingsArgs, $areaArgs, $transient);
    }

    private function _searchListingsByArea($listingsArgs, $areaArgs, bool $transient = false, $listingType = null)
    {
        if ($listingType === 'inactive') {
            $listingsArgs = $this->unsetUnusedArgsInInactiveListings($listingsArgs, $this->unsetArgsForInactiveListings, $this->statusesForInActiveListings);

            $listingType .= '/';
        } else {
            $listingType = '';
            $listingsArgs = $this->filteringRequestParams($listingsArgs);
        }

        $marketMeta = MarketMetadata::getInstance($listingsArgs['market'])->getMeta();
        $transientKey = md5("/ws/listings/{$listingType}search_/ws/areas/get" . '_' . json_encode($listingsArgs) . json_encode($areaArgs));

        if (isset($_GET['hjdebug']) && $_GET['hjdebug'] === '1') {
            delete_transient($transientKey);
        } elseif (!defined('HJI_CACHE_DISABLE') || (defined('HJI_CACHE_DISABLE') && HJI_CACHE_DISABLE === false)) {
            $listingsLastUpdate = -1;

            if (
                !empty($listingsArgs['market'])
                && $marketMeta
            ) {
                $listingsLastUpdate = $marketMeta->metadata->listings->lastUpdated;
            }

            $savedLastUpdate = get_option('hji_listings_lastUpdate', 0);
            $now = (new \DateTime('now'))->modify('-4 hour')->getTimestamp();

            if ($transient === true && ($savedLastUpdate === $listingsLastUpdate || $listingsLastUpdate < $now)) {

                if (($result = get_transient($transientKey)) !== false && isset($result['responses'][0]['result']['listings'])) {
                    return $result;
                }
            }
        }

        $params = [
            [
                'uri'        => "/ws/listings/{$listingType}search",
                'parameters' => $listingsArgs
            ],
            [
                'uri'        => "/ws/areas/get",
                'parameters' => $areaArgs
            ]
        ];

        $args = ['request' => json_encode($params)];

        $result = $this->makeAPIRequest("POST", "/ws/api/call", $args, [], $auth = false);


        if (!isset($result['responses'][0]['result']['listings']))
        {
            return false;
        }

        if ($transient === true) {
            ListingsTransient::setListingsLastUpdate($transientKey, $result, $marketMeta);
        }

        return $result;
    }

    public function searchInactiveListingsBySchoolAttendanceArea($listingsArgs, $areaArgs, $transient = false)
    {
        return $this->_searchListingsBySchoolAttendanceArea($listingsArgs, $areaArgs, $transient, 'inactive');
    }

    public function searchListingsBySchoolAttendanceArea($listingsArgs, $areaArgs, $transient = false)
    {
        return $this->_searchListingsBySchoolAttendanceArea($listingsArgs, $areaArgs, $transient);
    }

    private function _searchListingsBySchoolAttendanceArea($listingsArgs, $areaArgs, $transient = false, $listingType = null)
    {
        if ($listingType === 'inactive') {
            $listingsArgs = $this->unsetUnusedArgsInInactiveListings($listingsArgs, $this->unsetArgsForInactiveListings, $this->statusesForInActiveListings);

            $listingType .= '/';
        } else {
            $listingType = '';
            $listingsArgs = $this->filteringRequestParams($listingsArgs);
        }

        $params = [
            [
                'uri'        => "/ws/listings/{$listingType}search",
                'parameters' => $listingsArgs
            ],
            [
                'uri'        => "/ws/schools/attendance-areas/get",
                'parameters' => $areaArgs
            ]
        ];

        $args = ['request' => json_encode($params)];

        $result = $this->makeAPIRequest("POST", "/ws/api/call", $args, [], $auth = false);

        if (!isset($result['responses'][0]['result']['listings']))
        {
            return false;
        }

        return $result;
    }


    /**
     * Returns all listings for the currently selected market
     * 1st step for a full IDX app.
     *
     * @param [type] $args [description]
     * @return bool
     */
    function getListings($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/listings/get", $args, array(), $auth = false);

        if (!isset($result['listings']))
        {
            return false;
        }

        return $result['listings'];
    }


    function searchInactiveListings($args)
    {

        $args = $this->unsetUnusedArgsInInactiveListings($args, [], $this->statusesForInActiveListings);
        $listingType = apply_filters('ridx_past_sales_listing_type', $args);

        $args['listingType'] = $listingType;
        $result = $this->makeAPIRequest("GET", "/ws/listings/inactive/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getInactiveListings($args)
    {
        $args = $this->unsetUnusedArgsInInactiveListings($args, [], $this->statusesForInActiveListings);
        $result = $this->makeAPIRequest("GET", "/ws/listings/inactive/get", $args, array(), $auth = false);

        if (!isset($result['listings']))
        {
            return false;
        }

        return $result['listings'];
    }


    function searchSales($args)
    {
        $args = $this->filteringRequestParams($args);

        $result = $this->makeAPIRequest("GET", "/ws/sales/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getSales($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/sales/get", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function geocode($args)
    {
        $args = $this->filteringLicensedInStates($args);
        $result = $this->makeAPIRequest("GET", "/ws/address/geocode", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function standardize($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/address/standardize", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function updateLog($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/analytics/log", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function createUser($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/create", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function configureAnalytics($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/analytics/configure", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function logAction($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/analytics/log", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function searchActions($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/analytics/actions/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }

    function getActions($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/analytics/actions/get", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getAnalyticsStatus()
    {
        $result = $this->makeAPIRequest("GET", "/ws/analytics/status", array(), array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getUsers($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/get", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function updateUser($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/update", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function searchUsers($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function deleteUser($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/delete", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function addFavorites($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/favorites/add", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getFavorites($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/favorites/get", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function searchFavorites($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/favorites/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function removeFavorites($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/favorites/remove", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    /**
     * Adds user's saved search
     */
    function addSearches($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/searches/add", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    /**
     * Returns user saved search by search ID
     */
    function getSearch($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/searches/get", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }

    /**
     * Returns user's saved searches
     */
    function searchSearches($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/searches/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    /**
     * Deletes user's specified saved search.
     */
    function removeSearches($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/searches/remove", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    /**
     * Update user's specified saved search.
     */
    function updateSearches($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/users/searches/update", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    /**
     * Returns a specified school
     */
    function getSchool($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/schools/get", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    /**
     * Searches schools
     */
    function searchSchools($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/schools/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    /**
     * Returns automated valuation model (AVM) for a property.
     */
    function getAVM($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/avm", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getAgents($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/markets/agents/get", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function searchAgents($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/markets/agents/search", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }

    function getOffices($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/markets/offices/get", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function searchOffices($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/markets/offices/search", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function lookupAreas($args)
    {
        if (self::$areaBias && !isset($args['bias']))
        {
            $args['bias'] = self::$areaBias;
        }

        $result = $this->makeAPIRequest("GET", "/ws/areas/search", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getAreas($args)
    {
        if (self::$areaBias && !isset($args['bias']))
        {
            $args['bias'] = self::$areaBias;
        }

        $method = (count($args) < 100) ? 'GET' : 'POST';

        $result = $this->makeAPIRequest($method, "/ws/areas/get", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }

    public function searchAreasWithoutBias($args)
    {

        $result = $this->makeAPIRequest("GET", "/ws/areas/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getNeighborhoods($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/areas/neighborhoods/get", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function searchNeighborhoods($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/areas/neighborhoods/search", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getStates($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/areas/states/get", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getPlaces($args)
    {
        if (self::$areaBias == 'census-places')
        {
            return $this->getCensusPlaces($args);
        }

        return $this->getPostalCities($args);
    }


    /**
     * @param $args
     * @since 2.6.0
     *
     * @return array|bool|mixed|object
     */
    function getPostalCities($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/areas/postal-cities/get", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    /**
     * @param $args
     * @since 2.6.0
     *
     * @return array|bool|mixed|object
     */
    function getCensusPlaces($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/areas/census-places/get", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function searchCounties($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/areas/counties/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getCounties($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/areas/counties/get", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getZipCodes($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/areas/zipcodes/get", $args, array(), $auth = false);
        if ($result === false)
        {
            return false;
        }

        return $result;
    }

    function searchSchoolDistricts($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/schools/districts/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    //TODO: this method seem to be unused.
    // either make sure it's functional or get rid of it.
    /**
     * Returns markets schema
     * https://slipstream-test.homejunction.com/v1/docs/markets#get
     *
     * @param $args array - requires market key which can be string|array
     * @return bool|array of objects
     */
    function getMarketsMeta($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/markets/get", $args, array(), $auth = false);

        if (isset($result->result->markets) && $result->success)
        {
            return $result->result->markets;
        }

        return false;
    }


    /**
     * Returns lookup values for a given property
     * https://slipstream-test.homejunction.com/v1/docs/markets#enumerate
     *
     * @param $args
     * @return bool
     */
    function enumerate($args)
    {
        $args['distinct'] = true;
        $result = $this->makeAPIRequest("GET", "/ws/markets/enumerate", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function searchSessions($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/analytics/sessions/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function getSessions($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/analytics/sessions/get", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function searchMarkets($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/markets/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }

    /**
     * @param $latitude
     * @param $longitude
     * @return bool|mixed
     */
    public function marketsRank($latitude, $longitude)
    {
        $args = [
            'coords' => $latitude . ',' . $longitude,
        ];
        $result = $this->makeAPIRequest("GET", "/ws/markets/rank", $args, array(), $auth = false);

        return $result;
    }


    function searchPublicRecords($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/public-records/search", $args, array(), $auth = false);

        if ($result === false)
        {
            return false;
        }

        return $result;
    }


    function callWebService($webService)
    {
        if ($webService instanceof WebService)
        {
            $result = $this->makeAPIRequest("GET",
                "/ws/{$webService->getPath()}", $webService->getParameters(), array(), $auth = false);

            if ($result === false)
            {
                return false;
            }

            return $result;
        }
    }


    function searchNewConstructionListings($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/new-construction/listings/search", $args, array(), $auth = false);

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function getNewConstructionListings($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/new-construction/listings/get", $args, array(), $auth = false);

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function searchNewConstructionSubdivisions($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/new-construction/subdivisions/search", $args, array(), $auth = false);

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function getNewConstructionSubdivisions($args)
    {
        $result = $this->makeAPIRequest("GET", "/ws/new-construction/subdivisions/get", $args, array(), $auth = false);

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }

    /*
     * Makes the API call to the flexmls API.
     *
     * @param string $method HTTP method to use when making the call.  GET, POST, etc.
     * @param string $uri HTTP request URI to hit with the request
     * @param array $args array of key/value pairs of parameters.  added to request depending on HTTP method
     * @param array $caching array of caching settings. 'enabled' is true/false. 'minutes' defines how long if enabled
     * @return mixed Returns array of parsed JSON results if successful.  Returns false if API call fails
     */

    function makeAPIRequest($method, $uri, $args = array(), $data = array(), $is_auth_request = false, $a_retry = false)
    {
        $success = false;

        if (!$is_auth_request)
        {
            curl_setopt($this->ch, CURLOPT_HTTPHEADER,
                array("{$this->token_header}: {$this->last_token}"));
        }

        if (!is_array($args))
        {
            $args = array();
        }

        $http_parameters = http_build_query($args);

        $post_body = "";

        if ($method == "POST" && count($data) > 0)
        {
            // the request is to post some JSON data back to the API (like adding a contact)
            $post_body = $data;
        }

        // start putting the URL parts together

        $full_url = $this->api_base . $uri;

        if (!empty($http_parameters))
        {
            $full_url .= '?' . $http_parameters;
        }

        $request_headers = "";

        curl_setopt($this->ch, CURLOPT_URL, $full_url);
        curl_setopt($this->ch, CURLOPT_ENCODING, '');

        if ($method == "POST")
        {
            // put the built parameter key/values as the body of the POST request

            $request_headers .= "Content-Type: application/json\r\n";
            curl_setopt($this->ch, CURLOPT_POST, true);
            curl_setopt($this->ch, CURLOPT_POSTFIELDS, $post_body);
        }
        elseif ($method == "DELETE")
        {
            curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, "DELETE");
            curl_setopt($this->ch, CURLOPT_POSTFIELDS, $post_body);
            curl_setopt($this->ch, CURLOPT_FOLLOWLOCATION, 1);
            //curl_setopt($this->ch, CURLOPT_HEADER, 0);
        }
        else
        {
            curl_setopt($this->ch, CURLOPT_POST, 0);
            curl_setopt($this->ch, CURLOPT_HTTPGET, true);
        }

        // clear previous error messages
        $this->last_error_mess = null;
        $this->last_error_code = null;

        $response = curl_exec($this->ch);
        $info = curl_getinfo($this->ch);

        do_action('hji_api_request', $info, $response);

        @list($header, $response_body) = explode("\r\n\r\n", $response, 2);

        // Debug request

        $this->_debugRequest($post_body, $header, $response_body, $info, $uri);

        // Returns result or false

        $result = $this->_handleResponse($response_body, $info, $uri, $is_auth_request);

        if ($result == true)
        {
            return $result;
        }
        else if (($a_retry == false) && ($is_auth_request == false) && (($info['http_code'] == '403') || ($info['http_code'] == '404') || ($info['http_code'] == '500')))
        {
            $this->authenticate(true);
            $return = $this->makeAPIRequest($method, $uri, $args, $data, $is_auth_request, $a_retry = true);

            return $return;
        }
        else
        {
            return false;
        }

    }


    function _handleResponse($response_body, $info, $uri, $is_auth_request)
    {
        // Was connection successful

//        if ('200' == $info['http_code'])
//        {
//            $this->last_error_code = false;
//            $this->last_error_mess = false;
//        }
//        else
//        {
//            $result = json_decode(utf8_encode($response_body));
//
//            if (isset($result->error))
//            {
//                $this->setErrors($info['http_code'], $result->error->message);
//            }
//            else
//            {
//                $this->setErrors();
//            }
//
//            return false;
//        }

        // Start handling the response

        // Leave the following requests as objects

        $requests = array(
            "/ws/api/authenticate",
            "/ws/markets/get",
            "/ws/users",
            "/ws/analytics",
            "/ws/markets/search",
//            "/ws/sales/get",
//            "/ws/sales/search",
            "/ws/public-records/search",
            "/ws/new-construction/listings/get",
            "/ws/new-construction/listings/search",
            "/ws/new-construction/subdivisions/get",
            "/ws/new-construction/subdivisions/search",
        );

//        if (in_array($uri, $requests))
//        {
//            return $this->_handleObject($response_body, $info, $is_auth_request);
//        }

        foreach ($requests as $string)
        {
            if (stristr($uri, $string))
            {
                return $this->_handleObject($response_body, $info, $is_auth_request);
            }
        }

        return $this->_handleArray($response_body, $info, $is_auth_request);
    }


    function _handleArray($json, $info, $is_auth_request)
    {
        $result = false;
        $success = false;

        $response_body = json_decode(utf8_encode($json), true);

        if (isset($response_body['success']) && $response_body['success'] === true)
        {
            $success = true;
            $result  = $response_body['result'];
        }
        else if (isset($response_body['success']) && $response_body['success'] === false)
        {
            $success = false;

            $message = (isset($response_body['error']['message']))
                ? $response_body['error']['message']
                : '';

            $this->setErrors($info['http_code'], $message);
        }
        else
        {
            $this->setErrors();
        }


        if (isset($result['paging']))
        {
            $this->current_page = isset($result['paging']['number']) ? $result['paging']['number'] : 0;
            $this->page_size    = isset($result['paging']['size']) ? $result['paging']['size'] : 0;
            $this->total_pages  = isset($result['paging']['count']) ? $result['paging']['count'] : 0;
        }

        $this->last_count  = isset($result['total']) ? $result['total'] : 0;

        return $result;
    }


    function _handleObject($json, $info, $is_auth_request)
    {
        $result = false;
        $success = false;

        $response_body = json_decode(utf8_encode($json));

        if (is_object($response_body) && isset($response_body->success) && $response_body->success === true)
        {
            $success = true;
            $result = $response_body->result;
        }
        else if(is_object($response_body) && $response_body->success === false)
        {
            $success = false;

            $message = (isset($response_body->error->message))
                        ? $response_body->error->message
                        : '';

            $this->setErrors($info['http_code'], $message);
        }
        else
        {
            $this->setErrors();
        }


        if (isset($result->paging))
        {
            $this->current_page = isset($result->paging->number) ? $result->paging->number : 0;
            $this->page_size    = isset($result->paging->size) ? $result->paging->size : 0;
            $this->total_pages  = isset($result->paging->count) ? $result->paging->count : 0;
        }

        $this->last_count  = isset($result->total) ? $result->total : 0;

        return $response_body;
    }


    function _debugRequest($post_body, $header, $response_body, $info, $uri)
    {
        // for debugging

        $data = array(
            'webservice'      => $uri,
            'token'           => $this->last_token,
            'info'            => $info,
            'error'           => curl_error($this->ch),
            'post_body'       => $post_body,
            'response_header' => $header,
            'response'        => $response_body,
        );

        $this->debug_data[] = $data;

        if ($this->debug_mode == true)
        {
            fwrite($this->debug_log, $response_body . "\n");
        }

//        if ($this->debug_mode)
//        {
//            echo $response_body . "\n\n";
//        }
    }


    function setErrors($code = false, $message = false)
    {
        // Set errors manually
        // usually returned by the API and not CURL
        if ($code && $message)
        {
            $this->last_error_code = $code;
            $this->last_error_mess = $message;
        }

        // Set CURL errors

        else
        {
            $this->last_error_code =  curl_errno($this->ch);

            // curl_errno returns 0 if no error occured

            if ($this->last_error_code === 0)
            {
                $this->last_error_code = false;
                $this->last_error_mess = false;
            }
            else
            {
                // Try get error message from curl

                if ($message = curl_error($this->ch))
                {
                    $this->last_error_mess = $message;
                }

                // Otherwise see if we have message assigned to error code

                else if (isset($this->error_messages[$this->last_error_code]))
                {
                    $this->last_error_mess = $this->error_messages[$this->last_error_code];
                }

                // Last resort: we don't know what went wrong type of message.

                else
                {
                    $dump = (isset($this->debug_data) && !empty($this->debug_data)) ? json_encode($this->debug_data) : 'No debug data available.';
                    error_log('Responsive IDX Error: CURL returned an unknown error. JSON DUMP: [' . $dump .']');
                    $this->last_error_mess =  'Something went wrong. Please contact customer support.';
                }
            }
        }

        error_log('setErrors. ' . $code . ' - ' . $message . 'curl response: ' . json_encode(curl_getinfo($this->ch)));
    }


    /*
     * Take a value and clean it so it can be used as a parameter value in what's sent to the API.
     *
     * @param string $var Regular string of text to be cleaned
     * @return string Cleaned string
     */

    function clean_comma_list($var)
    {

        $return = "";

        if (strpos($var, ',') !== false)
        {
            // $var contains a comma so break it apart into a list...
            $list = explode(",", $var);
            // trim the extra spaces and weird characters from the beginning and end of each item in the list...
            $list = array_map('trim', $list);
            // and put it back together as a comma-separated string to be returned
            $return = implode(",", $list);
        }
        else
        {
            // trim the extra spaces and weird characters from the beginning and end of the string to be returned
            $return = trim($var);
        }

        return $return;
    }


    function exception_handler($exception)
    {
        echo "Uncaught exception: ", $exception->getMessage(), "\n";
    }


    /**
     * Outputs API request info for debugging purposes
     */
    function debug_data()
    {
        if (isset($this->debug_data) && isset($_GET['hjdebug']))
        {

            print '<textarea style="width: 600px; height: 600px">';
            print_r($this->debug_data);
            print '</textarea>';
        }
    }


    function _validateParams($args, $acceptedParams)
    {
        $tmp = $args;

        foreach ($tmp as $k => $v)
        {
            if (!in_array($k, $acceptedParams))
            {
                unset($args[$k]);
            }
        }

        return $args;
    }
}
