<?php
/**
 * Author URI: http://marinatanasov.com/
 * Plugin URI: https://github.com/tyxla/Gravity-Forms-Multiple-Form-Instances
 */

namespace hji\common\utils;

use GF_Fields;
use GFFormsModel;

/**
 * The main plugin class.
 */
class GravityFormsFilters
{
    const HJI_GF_MAPPING_OPTIONS = 'hji_gf_mapping';

    /**
     * @var array
     */
    private $formIdMapper = [];

    /**
     * Constructor.
     *
     * Used to initialize the plugin and hook the related functionality.
     *
     * @access public
     */
    public function __construct()
    {
        if (GFHelper::gravityFormsExists())
        {
            $this->addFilters();
        }
    }

    /**
     * Run our filters please
     */
    private function addFilters()
    {
        // hook the HTML ID string find & replace functionality
        add_filter('gform_get_form_filter', [$this, 'gform_get_form_filter'], 10, 2);
        add_filter('gform_footer_init_scripts_filter', [$this, 'gform_footer_init_scripts_filter'], 10, 2);
        add_filter('gform_form_post_get_meta', [$this, 'addFormIdField']);
        add_filter('gform_confirmation', [$this, 'customConfirmation'], 10, 4);
        add_filter('gform_pre_render', [$this, 'preRender'], 10, 1);
        add_filter('gform_pre_send_email', [$this, 'preSendEmail'], 10, 4);
    }

    public function preRender(array $form): array
    {
        if (!isset($form['fields']) || count($form['fields']) === 0) {
            return $form;
        }

        foreach ($form['fields'] as $key => $field) {
            if (isset($field['type']) && $field['type'] === 'honeypot') {
                $form['fields'][$key]['visibility'] = 'hidden';

                return $form;
            }
        }

        return $form;
    }

    /**
     * @param string $confirmation
     * @param array $form
     * @param array $entry
     * @param $ajax
     * @return string
     */
    public function customConfirmation($confirmation, $form, $entry, $ajax)
    {
        if (!isset($form['fields']) || count($form['fields']) === 0 ) {
            return $confirmation;
        }

        $newFormId = null;
        foreach ($entry as $value) {
            if (strpos($value, 'hji_form_id')  === 0) {
                $newFormId = str_replace('hji_form_id_', '', $value);
                break;
            }
        }

        if (empty($newFormId)) {
            return $confirmation;
        }

        $strings = [
            'gf_' . $form['id'] => 'gf_' . $newFormId,
            'gform_confirmation_wrapper_' . $form['id'] => 'gform_confirmation_wrapper_' . $newFormId,
            'gform_confirmation_message_' . $form['id'] => 'gform_confirmation_message_' . $newFormId,

        ];

        foreach ($strings as $find => $replace)
        {
            $confirmation = str_replace($find, $replace, $confirmation);
        }

        return $confirmation;
    }

    public function addFormIdField(array $form): array
    {
        if (
            !class_exists('\GFCommon')
            || \GFCommon::is_form_editor()
            || !class_exists('\GF_Fields')
            || !class_exists('\GFFormsModel')
        ) {
            return $form;
        }

        $flag = true;
        foreach ($form['fields'] as $field) {
            if ($field['hji_field_key'] === 'hji_form_id') {
                $flag = false;
            }
        }

        if ($flag) {
            $form['fields'][] = GF_Fields::create(
                [
                    'type' => 'text',
                    'id' => GFFormsModel::get_next_field_id( $form['fields'] ),
                    'label' => 'Form Id',
                    'formId' => $form['id'],
                    'pageNumber' => 1,
                    'visibility' => 'hidden',
                    'hji_field_key' => 'hji_form_id',
                    'inputName' => 'hji_form_id',
                    'allowsPrepopulate' => true,
                    'defaultValue' => 'hji_form_id_' . $form['id']
                ]
            );
        }

        return $form;
    }

    /**
     * Replaces all occurences of the form ID with a new, unique ID.
     *
     * This is where the magic happens.
     *
     * @access public
     *
     * @param string $form_string The form HTML string.
     * @param array  $form        Array with the form settings.
     * @return string $form_string The modified form HTML string.
     */
    public function gform_get_form_filter($form_string, $form)
    {
        // if form has been submitted, use the submitted ID, otherwise generate a new unique ID
        if (isset($_POST['gform_random_id'])) {
            $random_id = absint($_POST['gform_random_id']); // Input var okay.
        } else {
            $random_id = hexdec(uniqid());
        }

        if (empty($this->formIdMapper[$form['id']])) {
            $this->formIdMapper[$form['id']] = [];
        }

        $pregMatchResult = preg_match('/hji_form_id_\d{16}/', $form_string, $matches);
        if ($pregMatchResult === 1) {
            $matches = $matches[0];
            $randId = str_replace('hji_form_id_', '', $matches);

            if ($randId !== $form['id'] && $randId !== $random_id) {
                $random_id = $randId;
            }
        } else if ($pregMatchResult === false) {
            return $form_string;
        }

        $this->formIdMapper[$form['id']][$random_id] = false;

        $resultFormString = $this->applyFormFilter($form_string, $form['id'], $random_id, 'main body');

        return $resultFormString;
    }

    public function gform_footer_init_scripts_filter($form_string, $form)
    {
        $resultFormString = '';

        foreach ($this->formIdMapper[$form['id']] as $random_id => $isCalled) {
            if (!$isCalled) {
                $resultFormString .= $this->applyFormFilter($form_string, $form['id'], $random_id, 'footer-script');
                $this->formIdMapper[$form['id']][$random_id] = true;
            }

        }

        return $resultFormString;
    }

    /**
     * @param $form_string
     * @param $formId
     * @param $random_id
     * @return mixed
     */
    public function applyFormFilter($form_string, $formId, $random_id, $message = '')
    {
        $hidden_field = "<input type='hidden' name='gform_field_values'";

        // define all occurences of the original form ID that wont hurt the form input
        $strings = [
            ' gform_wrapper '                                               => ' gform_wrapper gform_wrapper_original_id_' . $formId . ' ',
            " id='gform_" . $formId                                     => " data-gform-id='" . $random_id . "' data-gform-original-id='" . $formId . "' id='gform_" . $random_id,
            "gchoice_" . $formId . '_'                                  => "gchoice_" . $random_id . '_',
            "for='choice_" . $formId . '_'                              => "for='choice_" . $random_id . '_',
            "id='choice_" . $formId . '_'                               => "id='choice_" . $random_id . '_',
            "id='label_" . $formId . '_'                                => "id='label_" . $random_id . '_',
            "'gform_wrapper_" . $formId . "'"                           => "'gform_wrapper_" . $random_id . "'",
            "'gf_" . $formId . "'"                                      => "'gf_" . $random_id . "'",
            "'gform_" . $formId . "'"                                   => "'gform_" . $random_id . "'",
            "'gform_ajax_frame_" . $formId . "'"                        => "'gform_ajax_frame_" . $random_id . "'",
            '#gf_' . $formId . "'"                                      => '#gf_' . $random_id . "'",
            "'gform_fields_" . $formId . "'"                            => "'gform_fields_" . $random_id . "'",
            "id='field_" . $formId . '_'                                => "id='field_" . $random_id . '_',
            'id="field_' . $formId . '_'                                => 'id="field_' . $random_id . '_',
            'value="hji_form_id_' . $formId                             => 'value="hji_form_id_' . $random_id,
            "value='hji_form_id_" . $formId                             => "value='hji_form_id_" . $random_id,
            "for='input_" . $formId . '_'                               => "for='input_" . $random_id . '_',
            "id='input_" . $formId . '_'                                => "id='input_" . $random_id . '_',
            "'gform_submit_button_" . $formId . "'"                     => "'gform_submit_button_" . $random_id . "'",
            '"gf_submitting_' . $formId . '"'                           => '"gf_submitting_' . $random_id . '"',
            "'gf_submitting_" . $formId . "'"                           => "'gf_submitting_" . $random_id . "'",
            '#gform_ajax_frame_' . $formId                              => '#gform_ajax_frame_' . $random_id,
            '#gform_wrapper_' . $formId                                 => '#gform_wrapper_' . $random_id,
            '#gform_' . $formId                                         => '#gform_' . $random_id,
            "trigger('gform_post_render', [" . $formId                  => "trigger('gform_post_render', [" . $random_id,
            'gformInitSpinner( ' . $formId . ','                        => 'gformInitSpinner( ' . $random_id . ',',
            "trigger('gform_page_loaded', [" . $formId                  => "trigger('gform_page_loaded', [" . $random_id,
            "'gform_confirmation_loaded', [" . $formId . ']'            => "'gform_confirmation_loaded', [" . $random_id . ']',
            'gf_apply_rules(' . $formId . ','                           => 'gf_apply_rules(' . $random_id . ',',
            'gform_confirmation_wrapper_' . $formId                     => 'gform_confirmation_wrapper_' . $random_id,
            'gforms_confirmation_message_' . $formId                    => 'gforms_confirmation_message_' . $random_id,
            'gform_confirmation_message_' . $formId                     => 'gform_confirmation_message_' . $random_id,
            'if(formId == ' . $formId . ')'                             => 'if(formId == ' . $random_id . ')',
            "window['gf_form_conditional_logic'][" . $formId . ']'      => "window['gf_form_conditional_logic'][" . $random_id . ']',
            "trigger('gform_post_conditional_logic', [" . $formId . ',' => "trigger('gform_post_conditional_logic', [" . $random_id . ',',
            'gformShowPasswordStrength("input_' . $formId . '_'         => 'gformShowPasswordStrength("input_' . $random_id . '_',
            "gformInitChosenFields('#input_" . $formId . '_'            => "gformInitChosenFields('#input_" . $random_id . '_',
            "jQuery('#input_" . $formId . '_'                           => "jQuery('#input_" . $random_id . '_',
            'gforms_calendar_icon_input_' . $formId . '_'               => 'gforms_calendar_icon_input_' . $random_id . '_',
            "id='ginput_base_price_" . $formId . '_'                    => "id='ginput_base_price_" . $random_id . '_',
            "id='ginput_quantity_" . $formId . '_'                      => "id='ginput_quantity_" . $random_id . '_',
            'gfield_price_' . $formId . '_'                             => 'gfield_price_' . $random_id . '_',
            'gfield_quantity_' . $formId . '_'                          => 'gfield_quantity_' . $random_id . '_',
            'gfield_product_' . $formId . '_'                           => 'gfield_product_' . $random_id . '_',
            'ginput_total_' . $formId                                   => 'ginput_total_' . $random_id,
            'GFCalc(' . $formId . ','                                   => 'GFCalc(' . $random_id . ',',
            'gf_global["number_formats"][' . $formId . ']'              => 'gf_global["number_formats"][' . $random_id . ']',
            'gform_next_button_' . $formId . '_'                        => 'gform_next_button_' . $random_id . '_',
            "id='gform_target_page_number_" . $formId                   => "id='gform_target_page_number_" . $random_id . '_' . $formId  ,
            "id='gform_source_page_number_" . $formId                   => "id='gform_source_page_number_" . $random_id . '_' . $formId  ,
            "aria-describedby='input_" . $formId . '_'                  => "aria-describedby='input_" . $random_id . '_' ,
            "id='gfield_description__"                                      => "id='gfield_description__" . $random_id . '_',
            "gfield_description_" . $formId . '_'                       => "gfield_description_" . $random_id . '_',
            "type='text' step='any' "                                       => "type='text' ",
            " type=\"text/css\""                                            => '',
            " type='text/javascript'"                                       => '',
            "gform_wrapper_original_id_" . $formId => "gform_wrapper_original_id_" . $random_id,
            "validation_message_" . $formId . "_" => "validation_message_" . $random_id . "_",
            'id="gform_' . $formId . '_validation_container"' => 'id="gform_' . $random_id . '_validation_container"',
            'data-js-reload="field_' . $formId . '_' => 'data-js-reload="field_' . $random_id . '_',
            $hidden_field => "<input type='hidden' name='gform_random_id' value='" . $random_id . "' />" . $hidden_field,
        ];

        // allow addons & plugins to add additional find & replace strings
        $strings = apply_filters('blvd/gform/multiple_instances_strings', $strings);

        // replace all occurences with the new unique ID
        foreach ($strings as $find => $replace)
        {
            $form_string = str_replace($find, $replace, $form_string);
        }

        return $form_string;
    }

    /**
     * @param $email
     * @param $message_format
     * @param $notification
     * @param $entry
     * @return mixed
     */
    public function preSendEmail($email, $message_format, $notification, $entry)
    {
        $mappingOptions = get_option(self::HJI_GF_MAPPING_OPTIONS, null);

        if ($mappingOptions === null || empty($mappingOptions['hji_responsive_idx:tour_this_home']['id'])) {
            return false;
        }

        $tourTheHomeFormId = $mappingOptions['hji_responsive_idx:tour_this_home']['id'];

        if ($entry['form_id'] != $tourTheHomeFormId) {
            return $email;
        }

        $dateLine = $timeLine = '';
        if (!empty($entry['2'])) {
            $date = (new \DateTime())::createFromFormat('Y-m-d', $entry['2']);
            $dateLine = 'Date: <span class="month">' . $date->format('M') . '</span>&nbsp;<span class="date">' . $date->format('d') . '</span>,&nbsp;<span class="day">' . $date->format('D') . '</span><br><br />';
        }

        if (!empty($entry['5'])) {
            $timeLine = 'Time: ' . $entry['5'] . '<br><br />';
        }

        $requestType = '';
        if (!empty($entry['15.1'])) {
            $requestType = $entry['15.1'];
        } elseif (!empty($entry['15.2'])) {
            $requestType = $entry['15.2'];
        }

        $message = '
            <html>
                <head>
                    <title>New submission from Tour This Home</title>
                </head>
                <body>
                    <br />
                    Tour Type: <i class="fa fa-user-o"></i> ' . $entry['1'] . '<br><br />
                    Name: ' . $entry['6'] . '<br><br />
                    Email: ' . $entry['8'] . '<br><br />
                    Phone: ' . $entry['7'] . '<br><br />'
                    . $dateLine
                    . $timeLine .
                    'Request Type: ' . $requestType . '<br><br />
                    Message: ' . $entry['9'] . '<br><br />
                    URL: ' . $entry['source_url'] . '<br />
                </body>
            </html>'
        ;

        $email['message'] = $message;

        return $email;
    }
}
