<?php
namespace hji\common\utils;


class WPHelper
{
    /**
     * Converts timestamp to WP Date format set in general settings
     *
     * @since 2.0
     * @param $timestamp
     * @return bool|string
     */
    static function toDate($timestamp)
    {
        $dateFormat = get_option('date_format');

        return self::toDateTimeFormat($timestamp, $dateFormat, null);
    }

    /**
     * Converts timestamp to WP Date and Time format
     * which is set in general settings
     *
     * @since 2.0
     * @param $timestamp
     * @return bool|string
     */
    public static function toDateTime($timestamp)
    {
        $dateFormat = get_option('date_format');
        $timeFormat = get_option('time_format');

        return self::toDateTimeFormat($timestamp, $dateFormat, $timeFormat);
    }

    /**
     * @param $timestamp
     * @param null/string $dateFormat
     * @param null/string $timeFormat
     * @return false|string
     */
    public static function toDateTimeFormat($timestamp, $dateFormat = null, $timeFormat = null)
    {
        $format = $dateFormat . ((empty($dateFormat) || empty($timeFormat)) ? '' : ' ') . $timeFormat;
        if (version_compare(get_bloginfo('version'), 5.3, '>=') && function_exists('wp_date')) {
            return wp_date($format, $timestamp, (new \DateTimeZone(self::getTimezoneString())));
        } else {
            self::setTimezone();

            return date_i18n($format, $timestamp);
        }
    }

    /**
     * @return bool
     */
    public static function is_heartbeat() {
        // Disregard WP AJAX 'heartbeat'call.  Why waste resources?
        if ( isset( $_POST ) && isset( $_POST['_nonce'] ) && wp_verify_nonce( sanitize_key( wp_unslash( $_POST['_nonce'] ) ), 'heartbeat-nonce' ) ) {
            if ( isset( $_POST['action'] ) && 'heartbeat' === sanitize_text_field( wp_unslash( $_POST['action'] ) ) ) {

                return true;
            }

            return false;
        }

        // Buh bye!
        return false;
    }

    /**
     * Execute to set timezone before using any date related functions.
     *
     * @since 2.0
     */
    static function setTimezone()
    {
        date_default_timezone_set(self::getTimezoneString());
    }

    /**
     * @return mixed|string|void
     */
    static function getTimezoneString()
    {
        if (version_compare(get_bloginfo('version'), 5.3, '>=') && function_exists('wp_timezone_string')) {
            $timezone = wp_timezone_string();
        } else {
            $timezone = get_option('timezone_string');
        }
        if (empty($timezone) || (!in_array($timezone, timezone_identifiers_list()) && !preg_match('/^[+-][0-9]{2}:[0-9]{2}$/', $timezone))) {
            $timezone = 'America/Los_Angeles';
        }

        return $timezone;
    }


    /**
     * Register a routine to be called when a plugin has been updated
     *
     * It works by comparing the current version with the version previously stored in the database.
     *
     * @since 2.0
     * @source https://core.trac.wordpress.org/attachment/ticket/14912/plugin-versions.4.diff
     *
     * @param string      $file A reference to the main plugin file
     * @param callback    $callback The function to run when the hook is called.
     * @param bool|string $version The version to which the plugin is updating.
     * @return bool
     */
    static function register_update_hook($file, $callback, $version = false)
    {
        if (!is_admin())
        {
            return;
        }

        if (!function_exists('is_plugin_active_for_network'))
        {
            require_once(ABSPATH . '/wp-admin/includes/plugin.php');
        }

        $plugin = plugin_basename($file);

        if (is_plugin_active_for_network($plugin))
        {
            $current_vers = get_site_option('active_plugin_versions', array());
            $network      = true;
        }
        elseif (is_plugin_active($plugin))
        {
            $current_vers = get_option('active_plugin_versions', array());
            $network      = false;
        }
        else
        {
            return false;
        }

        if (isset($current_vers[$plugin]) && version_compare($version, $current_vers[$plugin], '>'))
        {
            call_user_func($callback, $current_vers[$plugin], $network);

            $current_vers[$plugin] = $version;
        }

        if ($network)
        {
            update_site_option('active_plugin_versions', $current_vers);
        }
        else
        {
            update_option('active_plugin_versions', $current_vers);
        }
    }


    /**
     * Run on init action to make sure it's executed after
     * register_update_hook
     */
    static function registerPluginVersion($file)
    {
        if (!is_admin())
        {
            return;
        }

        if (!function_exists('is_plugin_active_for_network'))
        {
            require_once(ABSPATH . '/wp-admin/includes/plugin.php');
        }

        $plugin = plugin_basename($file);

        if (is_plugin_active_for_network($plugin))
        {
            $current_vers = get_site_option('active_plugin_versions', array());
            $network      = true;
        }
        elseif (is_plugin_active($plugin))
        {
            $current_vers = get_option('active_plugin_versions', array());
            $network      = false;
        }
        else
        {
            return false;
        }

        // Get version of the plugin

        $pluginData = get_plugin_data($file, true, true);
        $version = $pluginData['Version'];

        // If new pluginData shows newer version, then execute update hook

        if (!isset($current_vers[$plugin]) ||
            version_compare($version, $current_vers[$plugin], '>'))
        {
            $hook_args = array(
                'plugin'  => $file,
                'version' => $version,
                'currentVersion' => @$current_vers[$plugin]
            );

            do_action('hji_plugin_version_update' . $file, $hook_args);
            do_action('hji_plugin_version_update', $hook_args);
        }

        $current_vers[$plugin] = $version;

        if ($network)
        {
            update_site_option('active_plugin_versions', $current_vers);
        }
        else
        {
            update_option('active_plugin_versions', $current_vers);
        }
    }


    /**
     * Returns current URL loaded in the browser
     *
     * @return string
     */
    public static function getCurrentUrl()
    {
        $protocol = 'http://';

        if (isset($_SERVER['HTTPS']) && !empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off')
        {
            $protocol = 'https://';
        }

        $queryStr = filter_var($_SERVER['REQUEST_URI'], FILTER_SANITIZE_URL);
        $query = explode('?', $queryStr);

        if (isset($query[1]) && !empty($query[1]))
        {
            parse_str($query[1], $queryVars);
            
            $parts = explode('?', $_SERVER['REQUEST_URI']);
            $baseUrl = $parts[0];
            $queryVars = self::_sanitize_text_params($queryVars);
            $queryStr = http_build_query($queryVars);
            $requestUri = $baseUrl . '?' . $queryStr;
        }
        else
        {
            $requestUri = sanitize_text_field($_SERVER['REQUEST_URI']);
        }

        return "{$protocol}{$_SERVER['HTTP_HOST']}{$requestUri}";
    }


    private static function _sanitize_text_params(array $params)
    {
        foreach ($params as $k => $v)
        {
            if (is_string($v))
            {
                $params[$k] = sanitize_text_field($v);
            }
            else if (is_array($v))
            {
                $params[$k] = self::_sanitize_text_params($v);
            }
        }
        return $params;
    }

}
